#!/usr/bin/env python

import script_util as S
import redhawk.utils.get_ast as G

import optparse

usage = S.MakeStringFromTemplate("""
$prog add file1 ..

Add ASTs to the database.
(If a directory is given, it traverses it recursively.)
""")

def Main(args):
  parser = optparse.OptionParser(usage)
  options, args = parser.parse_args(args)

  if not len(args):
      parser.error("No files given.")

  database = S.GetDatabase()

  if database is None:
    S.ExitWithError(S.MakeStringFromTemplate(
        "No database found. Maybe $prog init first?"))

  ast_fetcher = G.CreateLASTFetcher(database)
  for f in S.GetSupportedFiles(args):
    ast = ast_fetcher.GetAST(f, key=S.GetKey(f, database))
    print "%s: Parsing "%(f)
  print "Adding to Database.."
  ast_fetcher.WriteDatabase()
  print "Done"
  return



#!/usr/bin/env python
""" Test the node_position module in redhawk/common"""

import redhawk.common.node_position as node_position

class TestNodePosition:
  def __init__(self):
    return

  def setUp(self):
    self.a = node_position.NodePosition("a", 10, 20)
    return

  def TestInit(self):
    """ Test NodePosition's init method."""
    assert(self.a!=None)
    return

  def TestNodePositionGetAttributes(self):
    """ Test NodePosition's GetFile, GetLine, GetColumn methods."""
    assert(self.a.GetFile() == "a")
    assert(self.a.GetLine() == 10)
    assert(self.a.GetColumn() == 20)
    return




#!/usr/bin/env python

""" Implementation of the XML Writer"""

import redhawk.common.node as N
import redhawk.common.types as T
import redhawk.utils.util as U
import writer

import lxml.etree as ET

def WriteToXML(tree):
  s = XMLWriter()
  return s.WriteTree(tree)


def WriteToFile(tree, filename):
  s = XMLWriter()
  s.GetTree(tree).write(filename)
  return


class XMLWriter(writer.Writer):
  def __init__(self):
    return

  def __AddChildToElement(self, child, parent_element):
    """ Adds a Child (which could itself be a list) as a child to the
    parent_element."""
    if  type(child) is list:
      group_node = ET.SubElement(parent_element, "ChildGroup")
      for c in child:
        self.__AddChildToElement(c, group_node)
    else:
      assert(isinstance(child, N.Node) or child == None)
      if child == None:
        print "We have a none child?"
        ET.SubElement(parent_element, "None")
      else:
        self.__ConvertToElement(child, parent_element)
    return

  def __AddPositionToElement(self, tree, parent_element):
    try:
      line, column, f = tree.position.GetLine(), tree.position.GetColumn(), tree.position.GetFile()
    except AttributeError, e:
      line, column, f = None, None, None

    attributes = {
        'line'    : str(line),
        'column'  : str(column),
        'file'    : str(f)
    }
    
    position_element = ET.SubElement(parent_element, "Position", attrib=attributes)
    return

  def __ConvertToElement(self, tree, parent_element):
    """ Converts a tree to an element tree, whose parent is `parent_element`."""
    attributes = dict([(key, val) for (key, val) 
      in tree.GetXMLAttributes()[1].items() if type(val) is str])

    print attributes
    print tree.GetName()
    node_element = ET.SubElement(parent_element, tree.GetName(), attrib=attributes)
    self.__AddPositionToElement(tree, node_element)
    self.__AddChildToElement(tree.GetChildren(), node_element)
    return


  def WriteTree(self, tree):
    """ Implementation of the base class method for writing the tree to a
        string."""
    return ET.tostring(self.GetTree(tree))

  def GetTree(self, tree):
    root = ET.Element("last")
    self.__ConvertToElement(tree, root)
    tree = ET.ElementTree(root)
    return tree



#!/usr/bin/env python

import redhawk.utils.get_ast as G

import ast
import pprint
import sys

filename = sys.argv[1]

pprint.pprint(ast.dump(
  G.GetLanguageSpecificTree(filename, 
                            database = None, 
                            language='python')))



#!/usr/bin/env python

""" Base Writer class."""

class Writer:
  def __init__(self):
    return

  def WriteTree(self, tree):
    raise NotImplementedError("WriteTree is not implemented in the base class.")

  def WriteTreeToFile(self, tree, filename):
    fp = open(filename, "w")
    fp.write(self.WriteTree(tree))
    fp.close()
    return





""" Test the util.py in redhawk/utils."""

import redhawk.utils.util as U

import nose.tools
import tempfile
import os

def TestConcat():
  """ Test Concat."""
  assert(U.Concat([[1], [2, 3], [4, 5]]) == [1, 2, 3, 4, 5])
  assert(U.Concat([[1], [2, 3], [4, 5]]) == [1, 2, 3, 4, 5])
  assert(U.Concat([[]]) == [])
  return


def TestFlatten():
  """ Test Flatten."""
  assert(U.Flatten([1, 2, 3, 4]) == [1, 2, 3, 4])
  assert(U.Flatten([[1], [2, 3], 4]) == [1, 2, 3, 4])
  assert(U.Flatten([[1], [[2, 3]], 4]) == [1, 2, 3, 4])

  assert(U.Flatten([[1], [[[2]]], 3]) == [1, 2, 3])


def TestGuessLanguageSuccess():
  """ Test Guess Langague for Success."""
  assert(U.GuessLanguage('foo.py') == 'python')
  assert(U.GuessLanguage('foo.c') == 'c')

  assert(U.GuessLanguage('foo.blah.py') == 'python')
  assert(U.GuessLanguage('foo.blah.c') == 'c')


def TestIfElse():
  """ Test IfElse construct."""
  # Test basic behaviour
  assert(U.IfElse(True, 2, 3) == 2)
  assert(U.IfElse(False, 2, 3) == 3)

  # Test lazy behaviour of condition
  assert(U.IfElse(lambda: True, 2, 3) == 2)
  assert(U.IfElse(lambda: False, 2, 3) == 3)

  # Test lazy behaviour of return values
  assert(U.IfElse(True, lambda: 2, lambda: 3) == 2)
  assert(U.IfElse(False, lambda: 2, lambda: 3) == 3)

  # Test all lazy behaviour
  assert(U.IfElse(lambda: True,
                lambda: 2,
                lambda: 3) == 2)


def TestFindFileInDirectoryOrAncestors():
  """ Test FindFileInDirectoryOrAncestors"""
  # Create an empty temp directory
  root_dir = tempfile.mkdtemp()
  a_dir = os.path.join(root_dir, "a")
  b_dir = os.path.join(a_dir, "b")

  # Create subdirectories
  os.mkdir(a_dir)
  os.mkdir(b_dir)

  # Create temporary file
  filepath = os.path.join(root_dir, "test_file")
  fp = open(filepath, "w")
  fp.close()
  print root_dir, a_dir, b_dir, filepath

  # Check if test_file can be found
  assert(U.FindFileInDirectoryOrAncestors("test_file", b_dir) == 
      filepath)

  # Ensure that adding /. to the path does not 
  # change the result of the test
  c_dir = os.path.join(b_dir, os.path.curdir)
  assert(U.FindFileInDirectoryOrAncestors("test_file", c_dir) == 
      filepath)

  # Change Permissions to 000 and ensure that an
  # IOError is thrown
  os.chmod(filepath, 0)
  raised = False
  try:
    U.FindFileInDirectoryOrAncestors("test_file", c_dir)
  except IOError, e:
    raised = True
  assert(raised)

  # Remove the file and temporary directories
  os.remove(filepath)
  assert(U.FindFileInDirectoryOrAncestors("test_file", b_dir) == 
     None)
  os.removedirs(b_dir)
  return






""" Test the code_generator_backend.py in redhawk/utils."""
import redhawk.utils.code_generator_backend as C

class TestCodeGeneratorBackend:
  def setUp(self):
    self.c = C.CodeGeneratorBackend()

  def TestWrite(self):
    """ Test CodeGeneratorBackend.Write."""
    self.c.Write("a")
    self.c.Write(" = ")
    self.c.Write("b")
    assert(self.c.GetCode() == "a = b")

  def TestGetCode(self):
    """ Test CodeGeneratorBackend.TestGetCode"""
    self.c.Write("a")
    assert(self.c.GetCode() == "a")

  def TestWriteLine(self):
    """ Test CodeGeneratorBackend.TestWriteLine"""
    self.c.WriteLine("print")
    self.c.WriteLine("print")
    assert(self.c.GetCode() == "print\nprint\n")

  def TestIndent(self):
    """ Test CodeGeneratorBackend.TestIndent"""
    self.c.WriteLine("def Blah():")
    self.c.Indent()
    self.c.WriteLine("a = 2")
    assert(self.c.GetCode() == "def Blah():\n  a = 2\n")

  def TestDedent(self):
    """ Test CodeGeneratorBackend.TestDedent"""
    self.c.Indent()
    self.c.WriteLine("a = 2")
    self.c.Dedent()
    self.c.WriteLine("return")
    assert(self.c.GetCode() == "  a = 2\nreturn\n")

  def TestEndToEnd(self):
    """ Test CodeGeneratorBackend.TestEndToEnd"""
    self.c.WriteLine("def Blah():")
    self.c.Indent()
    self.c.WriteLine("for i in range(3):")
    self.c.Indent()
    self.c.WriteLine("print i")
    self.c.Dedent()
    self.c.WriteLine("return 1")
    self.c.Dedent()
    assert(self.c.GetCode() == '\n'.join([
      "def Blah():",
      "  for i in range(3):",
      "    print i",
      "  return 1\n"]))



#!/usr/bin/env python
import script_util as S
import init
import add
import query
import prompt
import show

import optparse
import sys


HELP = S.MakeStringFromTemplate("""
Usage: $prog COMMAND [ARGS]

$prog - An AST based navigation system. 

The simplest use case is just:

  $ $prog query <query> files

Supported commands are:
  init        Create an EMPTY AST index.
  add         Add files to an AST index.
  query       Query for a pattern in a list of files, or in the index.
  prompt      Start a redhawk subshell, with preloaded modules, and parse-trees.
  show        Show (visualize) a file either as text, or as an image.

See $prog COMMAND --help for more detailed information about that command.

NOTE: The creation of an index for large projects is recommended. This can be
done with the init, and add commands.  Though recommended, an index is NOT
necessary.
""")

def Main():
  if len(sys.argv) < 2 or sys.argv[1] == "-h" or sys.argv[1] == "--help":
    print HELP
    sys.exit(0)

  dispatch = { 
      'add':   add.Main,
      'init':  init.Main,
      'query': query.Main,
      'prompt': prompt.Main,
      'show':  show.Main,
    }
  if sys.argv[1] in dispatch:
    dispatch[sys.argv[1]](sys.argv[2:])
  else:
    print "Command %s not found."%sys.argv[1]
    print HELP
    sys.exit(1)
  return


if __name__ == '__main__':
  Main()



#!/usr/bin/env python

import hashlib
import operator
import os
import sys
import traceback 


def AssertWithError(condition, error):
  """ If condition is false, exit with error."""
  if callable(condition):
    condition = condition()
  if not condition:
    ExitWithError(error)


def Concat(li):
  """ Concat :: [[a]] -> [a].
      similar to Haskell's concat."""
  return reduce(operator.concat, li)


def ExitWithError(error, backtrace=True):
  if backtrace:
    traceback.print_stack()
  sys.stderr.write("\n" + error + "\n")
  sys.exit(1)


def Flatten(li):
  """ Returns a generator that is a flattened out version of the original
      list.
      Example:
        Flatten([1, 2, [3, 4]])     ->  [1, 2, 3, 4]
        Flatten([[1, [2]], [3, 4]]) ->  [1, 2, 3, 4]
  """
  flat = []
  for x in li:
    if type(x) == list:
      flat.extend(Flatten(x))
    else:
      flat.append(x)
  return flat


def GetHashDigest(filename):
  """ Get the sha1 digest of `filename`)"""
  try:
    fp = open(filename)
    digest = hashlib.sha1(fp.read()).hexdigest()
    fp.close()
    return digest
  except IOError, e:
    sys.stderr.write(e)
    sys.exit(1)
  return


def GuessLanguage(filename):
  """ Attempts to Guess Langauge of `filename`. Essentially, we do a
  filename.rsplit('.', 1), and a lookup into a dictionary of extensions."""
  try:
    (_, extension) = filename.rsplit('.', 1)
  except ValueError:
    raise ValueError("Could not guess language as '%s' does not have an \
        extension"%filename)

  return {'c'   : 'c'
         ,'py'  : 'python'}[extension] 


def IfElse(condition, iftrue, iffalse):
  """ A re-implementation of C's ternary operator:
        Pass callables to avoid evaulation."""

  def CallIfPossible(x):
    if callable(x):
      return x()
    else:
      return x

  if CallIfPossible(condition):
    return CallIfPossible(iftrue)
  else:
    return CallIfPossible(iffalse)


def LogWarning(s, stream=sys.stderr):
  """ Log a warning to `stream` (default: sys.stderr.)"""
  stream.write("[WARNING]: %s\n"%(s))
  return


def StartShell(local_vars, banner=''):
  """ Start a shell, with the given local variables. It prints the given
  banner as a welcome message."""

  try:
    from IPython.Shell import IPShellEmbed
    ipshell = IPShellEmbed([])
    ipshell.set_banner(banner)
    ipshell(local_ns=local_vars)
  except ImportError:
    import code
    code.interact(local=local_vars, banner=banner)


def FindFileInDirectoryOrAncestors(filename, dirname, perm=os.R_OK | os.W_OK):
  """ Tries to find the file `filename` in the given directory `dirname` or
  its parents. When found it makes sure the permissions match the given
  `perm`. If no file is found, None is returned. If the file was found, but
  the permissions are not satisfied, it raises an IOError."""
  dirname = os.path.abspath(dirname)

  while not os.path.exists(os.path.join(dirname, filename)):
    parent_dirname = os.path.dirname(dirname)
    if dirname == parent_dirname:
      return None
    dirname = parent_dirname

  filepath = os.path.join(dirname, filename)
  if os.access(filepath, perm) is False:
    raise IOError("Read write permissions deny access to file %s"%(filepath))

  return filepath


def AdjustFilePathToBaseDirectory(filepath, base_dir):
  """ Adjust filepath, `filepath` to base directory, `base_dir`.
  This function for example, on inputs
    filepath = '/a/b/c/d.txt', base_dir = '/a/b'
  returns 'c/d.txt'

  If either the filepath nor the base_dir passed in is not in its absolutized
  normalized form, it is taken to be relative to the current directory
  (pwd)."""

  return os.path.relpath(filepath, base_dir)



#!/usr/bin/env python
import node
import redhawk.utils.util as U

class TreeConverter:
  """ The base tree converter class."""
  def __init__(self, filename=None):
    self.filename = filename
    return

  def ThrowNotImplementedError(self, tree):
    raise NotImplementedError("Convert%s not implemented."%(tree.__class__.__name__.capitalize()))


  def AttachParents(self, tree, parent = None):
    tree.SetParent(parent)
    for c in U.Flatten(tree.GetChildren()):
      if c is not None:
        try:
          self.AttachParents(c, tree)
        except AttributeError, e:
          print c, parent, tree
          raise AttributeError(e)
    return


  def Convert(self, tree):
    """ Calls ConvertTree, and attaches parent links using __AttachParents.
    This is the method to be called by outside methods, and functions."""
    l_ast = self.ConvertTree(tree)
    self.AttachParents(l_ast)
    return l_ast


  def ConvertTree(self, tree):
    method = "Convert" + tree.__class__.__name__.capitalize()
    visitor = getattr(self, method, self.ThrowNotImplementedError)
    return visitor(tree)



#!/usr/bin/env python
import redhawk.common.node as N
import redhawk.common.node_position as NP
import redhawk.common.tree_converter as tree_converter
import redhawk.common.types as T

import ast

# Map Python AST operators into the L-AST operators
# Add | Sub | Mult | Div | Mod | Pow | LShift 
#                  | RShift | BitOr | BitXor | BitAnd | FloorDiv
# 
# Eq | NotEq | Lt | LtE | Gt | GtE | Is | IsNot | In | NotIn
# And | Or 
BINARY_OPERATOR_CONVERSIONS = {
      'Add'       : 'ADD'
     ,'Sub'       : 'MINUS'
     ,'Mult'      : 'MULTIPLY'
     ,'Div'       : 'DIVIDE'
     ,'Pow'       : 'POWER'
     ,'BitXor'    : 'BITWISE_XOR'
     ,'BitOr'     : 'BITWISE_OR'
     ,'BitAnd'    : 'BITWISE_AND'
     ,'LShift'    : 'LSHIFT'
     ,'RShift'    : 'RSHIFT'
     ,'Mod'       : 'MOD'
     ,'FloorDiv'  : 'FLOOR_DIVIDE'
     ,'Lt'        : 'LT'
     ,'Gt'        : 'GT'
     ,'LtE'       : 'LTE'
     ,'GtE'       : 'GTE'
     ,'Eq'        : 'EQ'
     ,'NotEq'     : 'NOT_EQ'
     ,'And'       : 'BOOLEAN_AND'
     ,'Or'        : 'BOOLEAN_OR'
     ,'Is'        : 'IS'
     ,'In'        : 'IN'
     # ,'.'       : 'ATTRIBUTE_INDEX'
     # ,'->'      : 'ARROW' # TODO(spranesh): Bad Name?
}

NOT_BINARY_OPERATORS = {
      'NotIn'       : 'IN'
     ,'IsNot'       : 'IS'
}

# Invert | Not | UAdd | USub
UNARY_OPERATOR_CONVERSIONS = {
      'Invert' : 'BITWISE_NOT'
     ,'Not'    : 'BOOLEAN_NOT'
     ,'UAdd'   : 'UNARY_PLUS'
     ,'USub'   : 'UNARY_MINUS'
}
                              

class TransformCoord:
  """ A class to help transform coordinates from the python ast to the L-AST.
  This has been made a class, because it has state."""
  def __init__(self, filename):
    self.filename = filename
    return


  def GC(self, c):
    """ Get Coord."""
    try:
      return NP.NodePosition(self.filename, c.lineno, c.col_offset)
    except AttributeError, e:
      return None


def GetClassName(x):
  return x.__class__.__name__



class PythonTreeConverter(tree_converter.TreeConverter):
  def __init__(self, filename=None):
    tree_converter.TreeConverter.__init__(self, filename)
    self.gc = TransformCoord(filename)
    return


  def ConvertNonetype(self, tree):
    return None


  def ConvertExpr(self, tree):
    return self.ConvertTree(tree.value)


  def ConvertNum(self, tree):
    #TODO(spranesh): Should the value be really str-ed?
    return N.Constant(self.gc.GC(tree),
        value = str(tree.n),
        type = T.BaseType('number'))


  def ConvertStr(self, tree):
    #TODO(spranesh): Should the value be really str-ed?
    return N.Constant(self.gc.GC(tree),
        value = tree.s,
        type = T.BaseType('string'))


  def ConvertRepr(self, tree):
    return N.Show(position = self.gc.GC(tree),
        value = self.ConvertTree(tree.value))


  def ConvertExec(self, tree):
    return N.Exec(position = self.gc.GC(tree),
        body = self.ConvertTree(tree.body),
        locals = self.ConvertTree(tree.locals),
        globals = self.ConvertTree(tree.globals))


  def ConvertBinop(self, tree):
    """ Convert the BinOp(expr left, operator op, expr right) node.

      op\ in  Add | Sub | Mult | Div | Mod | Pow | LShift | RShift | BitOr
            | BitXor | BitAnd | FloorDiv"""
    return N.Expression(position = self.gc.GC(tree),
        operator = BINARY_OPERATOR_CONVERSIONS[GetClassName(tree.op)],
        children = map(self.ConvertTree, [tree.left, tree.right]))


  def ConvertName(self, tree):
    if isinstance(tree.ctx, ast.Param):
      return N.DefineVariable(self.gc.GC(tree),
          name = tree.id)
    return N.ReferVariable(self.gc.GC(tree),
        name = tree.id)


  def ConvertBoolop(self, tree):
    """ Convert the BoolOp(boolop op, expr* values) node. (And | Or) """
    return N.Expression(position = self.gc.GC(tree),
        operator = BINARY_OPERATOR_CONVERSIONS[GetClassName(tree.op)],
        children = map(self.ConvertTree, tree.values))


  def __ConvertBinaryOperation(self, op, position, left, right):
    """ We split the conversion into two cases - not in, is not (not
    operators), and the rest. In the not operators case, we use the
    NOT_BINARY_OPERATORS dictionary, and a BOOLEAN_NOT."""
    expr = N.Expression(position = position,
        operator = None,
        children = map(self.ConvertTree, [left, right]))

    if op in NOT_BINARY_OPERATORS:
      expr.operator = NOT_BINARY_OPERATORS[op]
      expr = N.Expression(position = position,
          operator = 'BOOLEAN_NOT',
          children = [expr])
                          
    else:
      expr.operator = BINARY_OPERATOR_CONVERSIONS[op]

    return expr


  def ConvertCompare(self, tree):
    """ Convert the Compare(expr left, cmpop* ops, expr* comparators) node.

    Each of the ops are \in   Eq | NotEq | Lt | LtE | Gt | GtE | Is 
                            | IsNot | In | NotIn

    More than one comparison is possible. Like 3 < x < 4.
    If the lenght of ops is one, then we return a simple comparison node, else
    we return an and of comparison nodes."""

    if len(tree.ops) is 1:
      return self.__ConvertBinaryOperation(position = self.gc.GC(tree),
          op = GetClassName(tree.ops[0]),
          left = tree.left, 
          right = tree.comparators[0])

    comparisons = []
    comparators = [tree.left] + tree.comparators
    for (i, c) in enumerate(tree.ops):
      comparisons.append(self.__ConvertBinaryOperation(
          position = self.gc.GC(comparators[i]),
          op = GetClassName(c),
          left = comparators[i],
          right = comparators[i+1]))

    return N.Expression(position = self.gc.GC(tree),
        operator = 'BOOLEAN_AND',
        children = comparisons)


  def ConvertUnaryop(self, tree):
    """ Convert the UnaryOp(unaryop op, expr operand) node.
    op \in Invert | Not | UAdd | USub """
    return N.Expression(position = self.gc.GC(tree),
        operator = UNARY_OPERATOR_CONVERSIONS[GetClassName(tree.op)],
        children = [self.ConvertTree(tree.operand)])


  def ConvertTuple(self, tree):
    """ Convert the Tuple(expr* elts, expr_context ctx) node."""
    return N.Tuple(position = self.gc.GC(tree),
        members = map(self.ConvertTree, tree.elts))


  def ConvertList(self, tree):
    """ Convert the List(expr* elts, expr_context ctx) node."""
    return N.List(position = self.gc.GC(tree),
        values = map(self.ConvertTree, tree.elts))

  def ConvertDict(self, tree):
    """ Convert the Dict(expr* keys, expr* values) node."""
    return N.Dict(position = self.gc.GC(tree),
        keys = map(self.ConvertTree, tree.keys),
        values = map(self.ConvertTree, tree.values))
 

  def ConvertAssign(self, tree):
    """ Convert the Assign(expr* targets, expr value) node."""
    if len(tree.targets) > 1:
      left = N.Tuple(position = self.gc.GC(tree.targets),
          members = map(self.ConvertTree, tree.targets))
    else:
      left = self.ConvertTree(tree.targets[0])

    return N.Assignment(position = self.gc.GC(tree),
        lvalue = left,
        rvalue = self.ConvertTree(tree.value))


  def ConvertAugassign(self, tree):
    """ Convert the AugAssign(expr target, operator op, expr value) node."""
    #TODO(spranesh): Not cheat?
    rvalue = N.Expression(position = self.gc.GC(tree.value),
        operator = BINARY_OPERATOR_CONVERSIONS[GetClassName(tree.op)],
        children = map(self.ConvertTree, [tree.target, tree.value]))

    return N.Assignment(position = self.gc.GC(tree),
        lvalue = self.ConvertTree(tree.target),
        rvalue = rvalue)


  def ConvertAttribute(self, tree):
    """ Convert the Attribute(expr value, identifier attr, expr_context ctx)
    node."""
    return N.Expression(position = self.gc.GC(tree),
        operator = 'ATTRIBUTE_INDEX',
        children = [self.ConvertTree(tree.value)
                   ,N.ReferVariable(self.gc.GC(tree), tree.attr)])


  def ConvertSlice(self, tree):
    """ Convert the Slice(expr? lower, expr? upper, expr? step) node."""
    return N.Slice(position = None,
                   lower = self.ConvertTree(tree.lower),
                   upper = self.ConvertTree(tree.upper),
                   step =  self.ConvertTree(tree.step))


  def ConvertExtslice(self, tree):
    """ Convert ExtSlice(slice *dims)."""
    #TODO(spranesh): Cheap hack. We only convert the first slice.
    #Maybe use a compound?
    return self.ConvertTree(tree.dims[0])


  def ConvertIndex(self, tree):
    """ Convert the Index(expr value) node."""
    return self.ConvertTree(tree.value)


  def ConvertSubscript(self, tree):
    """ Convert Subscript(expr value, slice slice, expr_context ctx) node."""
    return N.Expression(position = self.gc.GC(tree),
        operator = 'INDEX_INTO',
        children = [self.ConvertTree(tree.value),
                    self.ConvertTree(tree.slice)])


  def ConvertIf(self, tree):
    """ Convert the If(expr test, expr body, expr orelse) node."""
    return N.IfElse(position = self.gc.GC(tree),
        condition = self.ConvertTree(tree.test),
        if_true =  self.ConvertCompound(tree.body),
        if_false = self.ConvertCompound(tree.orelse))


  def ConvertIfexp(self, tree):
    """ Convert the IfExp(expr test, expr body, expr orelse) node."""
    return N.Expression(position = self.gc.GC(tree),
        operator = 'TERNARY_IF',
        children = map(self.ConvertTree, [tree.test, tree.body,
          tree.orelse]))


  def ConvertCompound(self, li):
    """ Convert a list of statements into a compound node. """
    assert(type(li) == list)
    if len(li) == 0:
      return None

    return N.Compound(position = self.gc.GC(li[0]),
        compound_items = map(self.ConvertTree, li))


  def ConvertModule(self, tree):
    return N.Module(position = NP.NodePosition(self.filename, 1, 1),
        filename = self.filename,
        children = map(self.ConvertTree, tree.body))


  def ConvertInteractive(self, tree):
    return N.Module(None,
        children = map(self.ConvertTree, tree.body))


  def ConvertExpression(self, tree):
    return self.ConvertInteractive(tree)


  def ConvertComprehension(self, tree):
    """ Convert the comprehension(expr target, expr iter, expr* ifs) node into
    a Generator.
    
    Note that the ifs must be combined into a single condition."""

    condition = None
    if tree.ifs != []:
      condition = N.Expression(position = self.gc.GC(tree.ifs[0]),
        operator = 'BOOLEAN_AND',
        children = map(self.ConvertTree, tree.ifs))

    return N.Generator(position = None,
        target = self.ConvertTree(tree.target),
        generator = self.ConvertTree(tree.iter),
        condition = condition)


  def __ConvertXComprehension(self, tree, elt, type):
    """ Convert a comprehension of type elt, whose LHS can be expressed as the
    converted expression elt. 
    
    (A helper function for ConvertListComprehension, ConvertDictComprehension,
    and ConvertSetComprehension.)"""

    return N.Comprehension(position = self.gc.GC(tree),
        expr = elt,
        type = type,
        generators = map(self.ConvertTree, tree.generators))


  def ConvertListcomp(self, tree):
    """ Conver the ListComp(expr elt, comprehension* generators) node.
    Type is 'list'."""

    return self.__ConvertXComprehension(tree,
        elt = self.ConvertTree(tree.elt),
        type = 'list')


  def ConvertGeneratorexp(self, tree):
    return self.__ConvertXComprehension(tree,
        elt = self.ConvertTree(tree.elt),
        type = 'generator')


  # Python 3
  def ConvertSetcomp(self, tree):
    """ Convert the SetComp(expr elt, comprehension* generators) node.
    Type is 'set'."""

    return self.__ConvertXComprehension(tree,
        elt = self.ConvertTree(tree.elt),
        type = 'set')


  # Python 3
  def ConvertDictcomp(self, tree):
    """ Convert the DictComp(expr key, expr value, comprehension* generators)
    node.
    
    Type is 'dict'.
    
    The value is stored as a pair expression."""
    elt = N.Tuple(position = self.gc.GC(tree),
        members = [self.ConvertTree(tree.key)
                  ,self.ConvertTree(tree.value)])

    return self.__ConvertXComprehension(tree,
        elt = elt,
        type = 'dict')
        

  def __ConvertArguments(self, args, position):
    """ Convert the Python argument node to the FunctionArguments L-AST node. The
    arguments need to be variable definitions, which do not exist elsewhere in
    Python (everything else is a ReferVariable).  We therefore handle it here
    itself. """

    arguments =  []
    for x in args.args:
      arguments.append(self.ConvertTree(x))


    for (i, y) in enumerate(args.defaults):
      arguments[len(arguments) - len(args.defaults) + i].init = self.ConvertTree(y)

    vararg, kwarg = None, None
    if args.vararg:
      vararg = N.DefineVariable(position = position, name = args.vararg)
    if args.kwarg:
      kwarg = N.DefineVariable(position = position, name = args.kwarg)


    return N.FunctionArguments(position = position,
                               arguments = arguments,
                               var_arguments = vararg,
                               kwd_arguments = kwarg)


  def ConvertFunctiondef(self, tree):
    """ Convert the FunctionDef node.
        FunctionDef(identifier name, arguments args, stmt* body, expr* decorator_list)

        arguments = (expr* args, identifier? vararg, identifier? kwarg, expr*
        defaults) 

        position, name, arguments, body
        """
    

    argument_node = self.__ConvertArguments(tree.args, self.gc.GC(tree))
    # Convert body
    body_node = N.Compound(position = self.gc.GC(tree),
                      compound_items = map(self.ConvertTree, tree.body))

    f = N.DefineFunction(position = self.gc.GC(tree),
                            name = tree.name,
                            arguments = argument_node,
                            body = body_node)


    return self.__EncapsulateNodeInDecorators(f, tree.decorator_list[::-1])


  def __EncapsulateNodeInDecorators(self, node, decorator_list):
    """ Encapsulate `node` in decorators from `decorator_list`"""
    for d in decorator_list[::-1]:
      node = N.FunctionDecorator(position = self.gc.GC(d),
                                    decorator = self.ConvertTree(d),
                                    function = node)
    return node
      

  def ConvertCall(self, tree):
    """ Convert a function call:

      Call(expr func, expr* args, keyword* keywords, expr? starargs, expr? kwargs)
      keyword = (identifier arg, expr value)

      into
      CallFunction: position, function, arguments """

    # Set up the arguments, using args, keywords, starargs, kwargs
    arguments = []
    for a in tree.args:

      arguments.append(self.ConvertTree(a))

    for k in tree.keywords:
      position_reference = tree.args[-1] if len(tree.args) > 0 else tree
      arguments.append(N.DefineVariable(position = self.gc.GC(position_reference),
                                        name = k.arg,
                                        init = self.ConvertTree(k.value)))


    argument_node = N.FunctionArguments(position = self.gc.GC(tree.args[0]) if len(tree.args) > 0 else None,
                                        arguments = arguments,
                                        var_arguments = self.ConvertTree(tree.starargs),
                                        kwd_arguments = self.ConvertTree(tree.kwargs))


    return N.CallFunction(position = self.gc.GC(tree),
                          function = self.ConvertTree(tree.func),
                          arguments = argument_node)


  def ConvertLambda(self, tree):
    """ Convert the lamdba function:
        Lambda(arguments args, expr body)

        into
        Lambda: position, arguments, value"""
    argument_node = self.__ConvertArguments(tree.args, self.gc.GC(tree))

    return N.Lambda(position = self.gc.GC(tree),
                    arguments = argument_node,
                    value = self.ConvertTree(tree.body))

  def ConvertFor(self, tree):
    """ Convert 

    For(expr target, expr iter, stmt* body, stmt* orelse)

    into the ForEach L-ASt node."""

    # TODO(spranesh): The target must be a DefineVariable rather than a ReferVariable.
    # So give it a context of Param(). (ConvertName will handle the rest).
    define_target_node = self.ConvertTree(tree.target)

    return N.ForEach(position = self.gc.GC(tree),
                     target = define_target_node,
                     iter_expression = self.ConvertTree(tree.iter),
                     body = self.ConvertCompound(tree.body))

  def ConvertWhile(self, tree):
    """ Convert

      While(expr test, stmt* body, stmt* orelse)

    into the While node."""
    return N.While(position = self.gc.GC(tree),
                   condition = self.ConvertTree(tree.test),
                   body = self.ConvertCompound(tree.body))

 
  def ConvertBreak(self, tree):
    """ Convert the Continue statement. (Duh?)"""
    return N.Break(position = self.gc.GC(tree))


  def ConvertContinue(self, tree):
    """ Convert the Continue statement. (Like, Duh?)"""
    return N.Continue(position = self.gc.GC(tree))


  def ConvertPass(self, tree):
    """ Convert the Pass statement. (Like, Duh-uh?)"""
    return N.Pass(position = self.gc.GC(tree))


  def ConvertAssert(self, tree):
    """ Convert Assert(expr test, expr? msg) """
    return N.Assert(position = self.gc.GC(tree),
                    test_expression = self.ConvertTree(tree.test),
                    message = self.ConvertTree(tree.msg))


  def ConvertDelete(self, tree):
    """ Convert the Delete(expr* targets) statement."""
    return N.Delete(position = self.gc.GC(tree),
                    targets = map(self.ConvertTree, tree.targets))

  def ConvertPrint(self, tree):
    """ Convert the Print(expr? dest, expr* values, bool nl)
      statement.
    """
    # TODO(spranesh): We currently ignore nl
    return N.Print(position = self.gc.GC(tree),
                   values = map(self.ConvertTree, tree.values),
                   stream = self.ConvertTree(tree.dest))


  def ConvertRaise(self, tree):
    """ Convert the Raise(expr? type, expr? inst, expr? tback)"""
    #TODO(spranesh): We currently ignore inst, and tback
    return N.Raise(position = self.gc.GC(tree),
                   exception_type = self.ConvertTree(tree.type))


  def ConvertExcepthandler(self, tree):
    """ Convert the ExceptHandler(expr? type, expr? name, stmt* body) node."""
    return N.ExceptionHandler(position = self.gc.GC(tree),
                              body = self.ConvertCompound(tree.body),
                              name = self.ConvertTree(tree.name),
                              type = self.ConvertTree(tree.type))


  def ConvertTryexcept(self, tree):
    """ Convert the 
    TryExcept(stmt* body, excepthandler* handlers, stmt* orelse)
    node. """
    return N.TryCatch(position = self.gc.GC(tree),
                      body = self.ConvertCompound(tree.body),
                      exception_handlers = map(self.ConvertTree, tree.handlers),
                      orelse = self.ConvertCompound(tree.orelse))


  def ConvertTryfinally(self, tree):
    """ Convert the | TryFinally(stmt* body, stmt* finalbody) node."""
    return N.Finally(position = self.gc.GC(tree),
                     body = self.ConvertCompound(tree.body),
                     final_body = self.ConvertCompound(tree.finalbody))


  def ConvertWith(self, tree):
    """ Convert With(expr context_expr, expr? optional_vars, stmt* body) node."""
    assign_node = ast.Assign(
        [tree.optional_vars],
        tree.context_expr,
        lineno = tree.lineno,
        col_offset = tree.col_offset)

    defvars = [self.ConvertTree(assign_node)]
    return N.Let(position = self.gc.GC(tree),
                 defvars = defvars, 
                 body = self.ConvertCompound(tree.body))

  def ConvertClassdef(self, tree):
    """ Convert the 
      ClassDef(identifier name, expr* bases, stmt* body, expr* decorator_list)
    node."""

    c = N.DefineClass(position = self.gc.GC(tree),
                      name = tree.name,
                      inherits = map(self.ConvertTree, tree.bases),
                      body = map(self.ConvertTree, tree.body))
    
    return self.__EncapsulateNodeInDecorators(c, tree.decorator_list[::-1])

  def ConvertAlias(self, tree):
    """ Convert alias = (identifier name, identifier? asname)"""
    return N.ModuleAlias(position = None, 
                         name = tree.name, 
                         asmodule = tree.asname)

  def ConvertImport(self, tree):
    """ Convert Import(alias* names) node."""
    return N.Import(position = self.gc.GC(tree),
                    import_aliases = map(self.ConvertAlias, tree.names))

  def ConvertImportfrom(self, tree):
    """ Convert the 
      ImportFrom(identifier? module, alias* names, int? level)
    node."""
    return N.ImportFrom(position = self.gc.GC(tree),
                        module = tree.module,
                        import_aliases = map(self.ConvertAlias, tree.names))

  def ConvertGlobal(self, tree):
    return N.ContextVariables(position = self.gc.GC(tree),
                     names = tree.names,
                     context = 'globals')

  def ConvertReturn(self, tree):
    return N.Return(position = self.gc.GC(tree),
                    return_expression = self.ConvertTree(tree.value))

  def ConvertYield(self, tree):
    return N.Yield(position = self.gc.GC(tree),
                    yield_expression = self.ConvertTree(tree.value))



#!/usr/bin/env python
""" This module contains procedures that are used to fetch Abstract Syntax
Trees, optionally using a database. The database is internally stored using
python pickle files.

There are two types of trees that are required.

1. The Language Agnostic Syntax Tree (LAST). This is the tree that redhawk
performs its operations on. This is what is required almost all of the time -
for querying, searching, editing (in upcoming versions), etc..

2. The various Language SPECIFIC trees. These are the trees emitted by the
parsers of the various languages - `pycparser`, the `ast` module in python,
etc.. These are used by Redhawk internally to CONSTRUCT the LAST. This is
almost never need by the API user. It is provided here since much of redhawk
internally requires it.

The ASTFetcher class is never instantiated directly. Instances of the
ASTFetcher class are created by calling either CreateLASTFetcher, or
CreateLanguageSpecificFetcher.

NOTE: The WriteDatabase method must be called MANUALLY when using the
ASTFetcher class to update the database file. This is so, for performance
reasons.


If a single tree is to be obtained, it is easier to call GetLAST, or
GetLanguageSpecificTree.

In short, only the GetLAST, and the CreateLASTFetcher functions are required
by most API users.

NOTE: None can be passed as the argument to the database argument, for any of
these functions. In such a case, it will simply fetch the required tree, by
parsing (and converting) it.
"""

import parse_ast
import redhawk
import util

import cPickle as P
import os
import sys

VERSION_KEY = '__redhawk__version__'


def GetLAST(filename, database, key=None, language=None):
  """ Get a Single LAST. Similar to creating a LAST fetcher instance using
  CreateLASTFetcher, and then using GetAST."""
  last_parser = lambda filename: parse_ast.GetLAST(filename, language)
  if database == None:
    return last_parser(filename)

  ast_fetcher = CreateLASTFetcher(database, language)
  ast = ast_fetcher.GetAST(filename, key)
  ast_fetcher.WriteDatabase()
  return ast



def GetLanguageSpecificTree(filename, database, key=None, language=None):
  """ Get a Single Language Specific Tree. Similar to creating a Language
  Specific Tree fetcher instance using CreateLanguageSpecificFetcher, and then
  using GetAST."""
  specific_parser = lambda filename: parse_ast.ParseFile(filename, language)
  if database is None:
    return specific_parser(filename)
  ast_fetcher = CreateLanguageSpecificFetcher(database, language)
  ast = ast_fetcher.GetAST(filename, key)
  ast_fetcher.WriteDatabase()
  return ast

def CreateLASTFetcher(database, language = None):
  """ A factory method for creating a LAST fetcher (It handles the parser
  argument).

  The `database_file` argument is a path to the database file.  If None is passed as the
  `database_file` argument, the trees are ALWAYS reparsed.

  The `language` argument can be set to 'c' or 'python' if the `database_file` will
  contain only c files or python files respectively. If a mixture is to be
  stored, this is set to None. (The `language` argument exists only for
  performance reasons -- it avoids language detection.)"""
  
  return ASTFetcher(
      database_file = database, 
      parser = lambda filename: parse_ast.GetLAST(filename, language))


def CreateLanguageSpecificFetcher(database, language=None):
  """ A factory method for creating a language specific tree Fetcher.

  The `database_file` argument is a path to the database file. If None is
  passed, the trees are ALWAYS reparsed.

  The `language` argument can be set to 'c' or 'python' if the database_file
  will contain only c files or python files respectively. If a mixture is to
  be stored, this is set to None. (The `language` argument exists only for
  performance reasons -- it avoids language detection.)"""
  return ASTFetcher(
      database_file = database,
      parser = lambda filename: parse_ast.ParseFile(filename, language))


class ASTFetcher:
  """ Fetches AST trees from a pickle file. 

  Normally, you would want to use either of the factory functions:
    * CreateLASTFetcher - Get an instance of the AST class, that fetches
      Language agnostic parse trees.

    * CreateLanguageSpecificFetcher - Get an instance of the AST class that
      fetches language specific parse trees.

  NOTE: The WriteDatabase must be called MANUALLY. This is so for performance
  reasons. If this is cumbersome, because you are fetching only one LAST or
  one Language Specific Tree, use the helper functions, `GetLAST` and
  `GetLanguageSpecificTree` respecitively."""

  def __init__(self, database_file, parser):
    self.database = database_file
    self.parser = parser
    self.changed = False
    # sys.stderr.write("Reading..\n")
    self.parsed_data = self.__ReadFromDatabase()
    # sys.stderr.write("Done.")
    return

  def __ReadFromDatabase(self):
    try:
      fp = open(self.database)
      parsed_data = P.load(fp)
      fp.close()
    except (IOError, EOFError):
      sys.stderr.write("Could not read from database.\n")
      parsed_data = {}
      self.changed = True
    except TypeError:
      parsed_data = {}
      self.changed = True

    # If version numbers don't match, don't read the data.
    if (not parsed_data.has_key(VERSION_KEY) or
        parsed_data[VERSION_KEY] != redhawk.__version__):
      parsed_data = {}
      parsed_data[VERSION_KEY] = redhawk.__version__
      self.changed = True
    return parsed_data


  def __WriteToDatabase(self):
    # sys.stderr.write("Changed?%s\n"%(self.changed))
    if self.database is None:
      return
    if self.changed == False:
      return

    try:
      fp = open(self.database, "w")
      P.dump(self.parsed_data, fp)
      fp.close()
    except IOError, e:
      sys.stderr.write("Could not write to Database: No Write permissions?\n")
    self.changed = False
    return

  def WriteDatabase(self):
    """ Update the databse file."""
    return self.__WriteToDatabase()


  def GetAST(self, filename, key=None):
    """ Extract a tree from the database. If `key` is None, the basename of
    the file is used. If the tree cannot be found, it is parsed, and stored in
    the database (in Memory).

    Note that to UPDATE the Database FILE, WriteDatabase() must be called.
    """
    digest = util.GetHashDigest(filename)
    key = key or os.path.basename(filename)
  
    if self.parsed_data.has_key(key):
      (pickled_digest, pickled_ast) = self.parsed_data[key]
      if pickled_digest == digest:
        return pickled_ast
  
    if self.parser is None:
      return None
  
    ast = self.parser(filename)
    self.parsed_data[key] = (digest, ast)
    self.changed = True
    return ast



#!/usr/bin/env python

""" Implementation of the CodeGeneratorBacked class."""

class CodeGeneratorBackend:
  """ A class to help code generation by managing indents."""
  def __init__(self, tab="  "):
    self.code = []
    self.tab = tab
    self.level = 0
    self.should_indent = True

  def GetCode(self):
    return "".join(self.code)

  def NewLine(self):
    self.code.append("\n")
    self.should_indent = True

  def Write(self, s):
    if self.should_indent:
      self.code.append(self.tab * self.level)
      self.should_indent = False
    self.code.append(s)
  
  def WriteLine(self, s):
    self.code.append(self.tab * self.level + s + "\n")

  def Indent(self):
    self.level = self.level + 1

  def Dedent(self):
    if self.level == 0:
      raise SyntaxError, "internal error in code generator"
    self.level = self.level - 1



#!/usr/bin/env python

import redhawk.common.node as N
import redhawk.common.tree_converter as T


"""
This module creates a test tree for us to run our code on.
Function Fact(n):
  Compound:
    if
      n == 1
      Compound
        return 1
      Compound
        return n * Fact(n-1)

When printed it represents:

(define-function Factorial 
  ((define-variable n))
  (compound 
    (if 
      (eq n 
        (constant 0))
      (compound 
        (compound 
          (return 
            (constant 1))))
      (compound 
        (compound 
          (return 
            (* n 
              (apply Factorial 
                ((- n 
                  (constant 1)))))))))))

"""


if_true = N.Compound(
    position = None,
    compound_items = [
      N.Return(
        position = None,
        return_expression = N.Constant(
          position = None,
          value = '1'))])

fact_arguments = N.FunctionArguments(
    position = None,
    arguments = [
      N.Expression(
        position = None,
        operator = 'MINUS',
        children = [
          N.ReferVariable(
            position = None,
            name = 'n'),
          N.Constant(
            position = None,
            value = '1')])])



if_false = N.Compound(
    position = None,
    compound_items = [
      N.Return(
        position = None,
        return_expression = N.Expression(
          position = None,
          operator = 'MULTIPLY',
          children = [
            N.ReferVariable(
              position = None,
              name = 'n'),
            N.CallFunction(position = None,
              function = N.ReferVariable(
                position = None,
                name = 'Factorial'),
              arguments = fact_arguments)]))])

ifelse = N.IfElse(
    position = None,
    condition = N.Expression(
        position = None,
        operator = 'EQ',
        children = [
            N.ReferVariable(
               position = None,
               name = 'n'),
            N.Constant(
               position = None,
               value = '0')]),
    if_true = N.Compound(
        position = None,
        compound_items = [if_true]),
    if_false = N.Compound(
        position = None,
        compound_items = [if_false]))


factorial_tree = N.DefineFunction(
     position = None,
     name = 'Factorial',
     arguments = N.FunctionArguments(
          position = None,
          arguments = [
              N.DefineVariable(
                   position = None
                   ,name = 'n')]),
     body = N.Compound(
          position = None,
          compound_items = [ifelse]))


T.TreeConverter().AttachParents(factorial_tree)
tree = factorial_tree

if __name__ == '__main__':
  print factorial_tree.ToStr()



#!/usr/bin/env python
""" Test initialising of the Node class in redhawk/common/node.py"""
import redhawk.common.node as node

def TestNodeInit():
  """ Test Node cannot be initialised. """
  try:
    n = node.Node()
  except NotImplementedError, e:
    return
  assert(False)




""" Type Classes.
    
    Used to structure the Type Heirarchy.
    
    This file is AUTO GENERATED from
      _types_cfg.yaml using _ast_gen.py
    The header is stored in _types_header.py
"""

import node

class Type(node.Node):
  """ The Base Type class."""
  pass



class Array(Type):
  """Represents an array type."""
  def __init__(self, array_type):
    self.array_type = array_type
    return

  def GetChildren(self):
    li = []
    li.append(self.array_type)
    return li

  def GetSExp(self):
    li = []
    li.append('array-of')
    li.append(self.array_type)
    return li



class BaseType(Type):
  """Represents a base type (stored as a string)."""
  def __init__(self, base_type):
    self.base_type = base_type
    return

  def GetSExp(self):
    li = []
    li.append('base-type')
    li.append(self.base_type)
    return li



class EnumeratorType(Type):
  """Represents an enumerator type."""
  def __init__(self, enumerator_type):
    self.enumerator_type = enumerator_type
    return

  def GetSExp(self):
    li = []
    li.append('enum-of-type')
    li.append(self.enumerator_type)
    return li



class Pointer(Type):
  """Represents a Pointer-to type."""
  def __init__(self, ptr_type):
    self.ptr_type = ptr_type
    return

  def GetChildren(self):
    li = []
    li.append(self.ptr_type)
    return li

  def GetSExp(self):
    li = []
    li.append('pointer-to')
    li.append(self.ptr_type)
    return li



class StructureType(Type):
  """Represents a Structure type."""
  def __init__(self, structure_type):
    self.structure_type = structure_type
    return

  def GetSExp(self):
    li = []
    li.append('structure-of')
    li.append(self.structure_type)
    return li



class UnionType(Type):
  """Represents a Union type."""
  def __init__(self, union_type):
    self.union_type = union_type
    return

  def GetSExp(self):
    li = []
    li.append('union-of')
    li.append(self.union_type)
    return li





#!/usr/bin/env python

""" Test Python Conversions."""

import redhawk.python.python_tree_converter as P
import redhawk.utils.get_ast as G

import nose.tools

import ast
import glob
import os
import sys

RELATIVE_TEST_PATH = "test/files/python"
  
def SetUp(filename, rel_path=RELATIVE_TEST_PATH):
  """ SetUp returns a parsed python Program."""
  PICKLE_FILE = "test/files/asts_python.pickle"
  return G.GetLanguageSpecificTree(os.path.join(rel_path, filename),
      PICKLE_FILE,
      language='python')


def ConvertTree(t, filename=None, verbose=True):
  """ Convert the Python-AST into the L-AST."""
  if verbose:
    print ast.dump(t)
  c = P.PythonTreeConverter(filename)
  a = c.Convert(t)
  if verbose:
    print a.ToStr(), "\n\n"
  return a


def ReadLines(filename):
  return open("%s/%s"%(RELATIVE_TEST_PATH, filename)).readlines()


def CTWD(filename, description, backquote = True, verbose = True):
  """ Return the ConvertTree function after setting its description tag
  appropriately."""
  x = lambda t: ConvertTree(t, filename, verbose)
  if backquote:
    x.description = "Test [%s]:`%s`"%(filename, description.strip())
  else:
    x.description = "Test [%s]:%s"%(filename, description.strip())
  return x



def TestExpressions():
  filename = "expressions.py"
  t = SetUp(filename)
  test_descriptions = ReadLines(filename)[1:]

  for i in range(len(t.body)):
    yield CTWD(filename, test_descriptions[i]), t.body[i]


def TestStatements():
  filename = "statements.py"

  t = SetUp(filename)
  test_descriptions = ReadLines(filename)[1:]
  for i in range(len(t.body)):
    yield CTWD(filename, test_descriptions[i]), t.body[i]


def TestAllPrograms():
  filenames = glob.glob(os.path.join(RELATIVE_TEST_PATH, "*.py"))
  filenames.sort()

  for f in filenames:
    description = open(f).readline().strip()
    if description[0] == '#':
      description = description[1:].strip()
    else:
      description = "---"

    t = SetUp(f, rel_path = "")
    yield CTWD(f, description, backquote=False),  t



#!/usr/bin/env python
import script_util as S
import redhawk.common.selector as selector
import redhawk.common.position as position
import redhawk.utils.get_ast as G
import redhawk.utils.util as U

import optparse
import os
import sys
import tempfile
import webbrowser

usage = S.MakeStringFromTemplate("""$prog prompt [options] file1 file2 ..

The prompt command is used to explore the ASTs, or for more complex queries
using the selector API.

Type help() at the prompt to know more.
""")

USE_DATABASE = False


def ConvertFileToAST(f):
  """ Convert a file into an language agnostic AST."""
  if USE_DATABASE:
    database = S.GetDatabase()
    return G.GetLAST(f, database, key=S.GetKey(f, database))
  else:
    return G.GetLAST(f, database=None)

def ConvertCodeToAST(s, language):
  """ Convert a code snippet into an language agnostic AST."""
  name = tempfile.mktemp(suffix='.' + language)
  fp = open(name, "w")
  fp.write(s)
  fp.close()
  return ConvertFileToAST(name)


def Main(args):
  parser = optparse.OptionParser(usage)
  parser.add_option(
      "-n",
      "--no-database",
      action="store_false",
      dest="use_db",
      default=True,
      help = "Explicity tell redhawk to NOT use the database." + S.OPTIONS_DEFAULT_STRING)

  options, args = parser.parse_args(args)

  database = None
  if options.use_db:
    database = S.GetDatabase()
    USE_DATABASE = True

  if args is []:
    trees = []
  else:
    ast_fetcher = G.CreateLASTFetcher(database)
    trees = [ast_fetcher.GetAST(f, key=S.GetKey(f, database)) for f in args]
    ast_fetcher.WriteDatabase()
  return EnterShell(trees)


def Help(display=True):
  s = """

Built in Variables:
    trees - contains the parse trees of the files passed in the command line

Built in Functions:
    ConvertFileToAst - Converts a file into a language agnostic AST.
    ConvertCodeToAst - Converts a code snippet into a language agnostic AST.
    Help             - Displays this prompt.
    ShowASTAsImage   - Shows the AST as a graph using dot.

Built in Modules:
    S - redhawk.common.selector 
    P - redhawk.common.position 

To view this again, use the Help function.  """

  if display:
    print s
  else:
    return s
  

def EnterShell(trees):
  local_vars = {
      'trees' : trees,
      'ConvertFileToAST': ConvertFileToAST,
      'ConvertCodeToAST': ConvertCodeToAST,
      'Help':Help,
      'ShowASTAsImage':S.ShowASTAsImage,
      'S':selector,
      'P':position,
    }
  U.StartShell(local_vars, banner=Help(display=False))


if __name__ == '__main__':
  Main()



# Test Statements
a = b + c
(a, b) = b, a+b
b += 1
c *= 100
for i in range(100): sum+=i*2
while i < 100: i += 1
assert(True == True)
assert(True != False, "Python doesn't understand True and False!")
del li
del li[2]
print a
print a, b, c
print>>sys.stderr, a, b, c
def f(a): pass
raise AttributeError("Oh noes! Attribute Error!")
global x, y
exec 2 + 3



#!/usr/bin/env python

""" Test Expressions """

import c_test_utils as CT

TEST_FILE = "expressions.c"

# Leave out the first two lines void f(),  { and the last }
test_descriptions = open("%s/%s"%(CT.RELATIVE_TEST_PATH,
  TEST_FILE)).readlines()[2:-1]

ConvertTree = CT.ConvertTree

def TestExpressions():
  t = CT.SetUp(TEST_FILE)
  for (i, description) in enumerate(test_descriptions):
    ConvertTree.description = "Test `%s`"%(description.strip())
    yield ConvertTree, t.children()[0].body.block_items[i]



#!/usr/bin/env python

""" An xpath like query system.


XPathQuery = AtomicQuery ('/' AtomicQuery)*

AtomicQuery = LocationQuery
            | [LocationQuery]     # ChildNodeMatchQuery query

LocationQuery =   .
                | ..
                | *
                | **
                | NodeQuery


NodeQuery = identifier? @[identifier=string]* @{codeblock}? [number]

"""
import _selector
import traverse

import redhawk.utils.parser_combinator as P
import redhawk.utils.util as U

import itertools
import re
import sys

# All our filter functions are from sequences to sequences
#   Filter :: Iterable -> Iterable

def Children(it):
  """ Return a generator to the children of all the nodes in the passed
  iterable."""
  for n in it:
    for c in U.Flatten(n.GetChildren()):
      if c is not None:
        yield c

def Parents(it):
  """ Return a generator to the parents of all the nodes in the passed
  iterable. Note that we first use a set to remove duplicates among the
  parents."""
  return iter(set((n.GetParent() for n in it)))


class Query:
  def ToStr(self): return ''

  def Filter(self):
    raise NotImplementedError("Not Implemented in base class!")

  def __repr__(self):
    s = self.ToStr()
    if s: return "%s->%s"%(self.__class__.__name__, s)
    return self.__class__.__name__

  def __str__(self):
    return self.__repr__()


class DotQuery(Query):
  def Filter(self, it):
    return it


class DotDotQuery(Query):
  def Filter(self, it):
    return Parents(it)


class StarQuery(Query):
  def Filter(self, it):
    return Children(it)


class StarStarQuery(Query):
  def Filter(self, it):
    """ We do not want duplicates. This means storing all the objects into a
    set. This is an expensive operation in memory, and in time O(n log n). We
    are doing this for now, but probably need to come up with (heuristic
    ways?) later."""
    iterators = [traverse.DFS(i) for i in it]
    return iter(set(itertools.chain(*iterators)))


class NodeMatchQuery(Query):
  # By default variables are '', if not found
  def __init__(self, node_type, attributes, codegroup, position):
    self.node_type = node_type or None
    self.attributes = dict(attributes)
    self.codegroup = codegroup or None
    self.position = position or None

    self.function = None
    if self.codegroup:
      try:
        self.function = eval('lambda n: '+ self.codegroup, {}, {})
      except StandardError, e:
        raise SyntaxError(str(e) + ": " + self.codegroup)
    return

  def Filter(self, it):
    it = Children(it) # The children of the given nodes are checked.
    s = _selector.Selector(node_type = self.node_type,
                           function = self.function,
                           **self.attributes)
    matched_nodes = itertools.ifilter(s, it)
    if self.position is not None:
      matched_nodes = list(matched_nodes)
      if self.position < len(matched_nodes):
        return iter([matched_nodes[self.position]])
      else:
        return iter([])
    return matched_nodes

  def ToStr(self):
    return "node_type = %s, attributes = %s, codegroup = %s, position = %s"%(
        self.node_type , self.attributes, self.codegroup, self.position)


class ChildNodeMatchQuery(Query):
  def __init__(self, q):
    assert(isinstance(q, NodeMatchQuery))
    self.child_query = q

  def ToStr(self): return "Child: " + self.child_query.ToStr()

  def Filter(self, it):
    # Query the children. Since child_query is a NodeMatchQuery, it
    # automatically checks the children.
    filtered_children = self.child_query.Filter(it)
    parents_of_filtered_children = Parents(filtered_children)
    return parents_of_filtered_children

# Parse xpath
reg_identifier = re.compile(r"[a-zA-Z_][a-zA-Z0-9_]*")
reg_string     = re.compile(r"'[^']*'|" + r'"[^"]*"')
reg_codeblock  = re.compile(r"@{[^}]*}")
reg_number     = re.compile(r"-?[0-9]+")

# Warn people of Empty Codeblocks
def CleanAndWarnEmptyCodeBlock(s):
  r = s[2:-1]
  if not r: raise SyntaxError("Empty Code block found.")
  return r

string_parser = P.Clean(P.Regex(reg_string), lambda x: x[1:-1])
codeblock_parser = P.Clean(P.Regex(reg_codeblock), CleanAndWarnEmptyCodeBlock)
identifier_parser = P.Regex(reg_identifier)
number_parser = P.Clean(P.Regex(reg_number), lambda x: int(x))

attr_match_parser = P.Clean(
  P.Sequence(
    (P.Literal("@["), None),
    (identifier_parser, "Identifier expected"),
    (P.Literal("="), "'=' expected"),
    (string_parser, "String expected"),
    (P.Literal("]"), "Closing ']' expected")),
  lambda x: (x[1], x[3]))

position_parser = P.Clean(
    P.Sequence(
      (P.Literal("["), None),
      (number_parser, "Number expected"),
      (P.Literal("]"), "Closing ']' expected")
      ),
    lambda x: x[1])


def CleanAndWarnEmptyNodeQuery(li):
  assert(len(li) == 4)
  for x in li:
    if x:
      return NodeMatchQuery(*li)
  raise SyntaxError("Invalid or Empty NodeQuery!")

node_query_parser = P.Clean(
  P.Sequence(
    (P.Maybe(identifier_parser), None),
    (P.Maybe(P.OnePlus(attr_match_parser)), None),
    (P.Maybe(codeblock_parser), None),
    (P.Maybe(position_parser), None)),
  CleanAndWarnEmptyNodeQuery)

dot_parser = P.Clean(P.Literal("."), lambda x: DotQuery())
dotdot_parser = P.Clean(P.Literal(".."), lambda x: DotDotQuery())
star_parser = P.Clean(P.Literal("*"), lambda x: StarQuery())
starstar_parser = P.Clean(P.Literal("**"), lambda x: StarStarQuery())

location_query_parser = P.Choice(
    dotdot_parser,
    starstar_parser,
    dot_parser,
    star_parser,
    node_query_parser)

child_node_match_parser = P.Clean(
  P.Sequence(
    (P.Literal("["), None),
    (node_query_parser, "Expected a node query"),
    (P.Literal("]"), "Expected closing ']'")),
  lambda x: ChildNodeMatchQuery(x[1]))

atomic_query_parser = P.Choice(
    child_node_match_parser,
    location_query_parser)

slash_sep_atomic_queries_parser = P.OnePlus(
  P.Clean(
    P.Sequence(
        (P.Literal("/"), None),
        (atomic_query_parser, "Invalid Atomic Query.")),
    lambda x: x[1]))
      
xpath_query_parser = P.Clean(
  P.Sequence(
    (atomic_query_parser, "Invalid Atomic Query"),
    (P.Maybe(slash_sep_atomic_queries_parser, lambda: list()), None),
    (P.Finished(), "Left over munchies! Or maybe too few. Who's to know?")),
  lambda x: [x[0]] + x[1])

def ParseXPath(s):
  """ Parse an XPath Query."""
  if s[0] == '/':
    raise SyntaxError("Queries should not start with a '/'")
  if s[-1] == '/':
    raise SyntaxError("Queries should not end with a '/'")
  try:
    return xpath_query_parser(s)[0]
  except SyntaxError, e:
    print "Syntax Error: %s"%(e)
    sys.exit(1)
  return

def ApplyParsedXPathQuery(trees, xpath_query):
  """ Apply a parsed XPath Query to a list (sequence) of `trees`."""
  assert(type(xpath_query) == list)
  filtered_nodes = iter(trees[:])
  for q in xpath_query:
    filtered_nodes = q.Filter(filtered_nodes)
  return list(filtered_nodes)

def XPath(trees, xpath_string):
  """ Parse and apply the xpath query in `xpath_string` to the list (sequence)
  of `trees`."""
  return ApplyParsedXPathQuery(trees, ParseXPath(xpath_string))


def Main():
  """ Only for testing."""

  import position
  import redhawk.utils.get_ast as G

  if len(sys.argv) < 2:
    print """ %s <xpath-query> [files].

    If files are not given, the parsed xpath-query is printed.
    Otherwise the concerned lines in each file is printed."""%(sys.argv[0])
    sys.exit(1)

  parsed_query = ParseXPath(sys.argv[1])
  if len(sys.argv) == 2:
    print parsed_query
    return

  for f in sys.argv[2:]:
    ast = G.GetLAST(f, database = None)
    results = list(ApplyParsedXPathQuery([ast], parsed_query))
    print "+++ %s"%(f)
    for r in results:
      print position.ShowPosition(position.GetPosition(r))
    print
  return

if __name__ == '__main__':
  Main()



#!/usr/bin/env python
""" Implementation of redhawk show"""
import script_util as S

import redhawk.utils.get_ast as G

import optparse
import os

usage = S.MakeStringFromTemplate("""$prog show [options] <query> <file>

This command is used to show an AST. An AST can either be printed (default),
or converted to an image (this requires python-graphviz).
""")


def Main(args):

  parser = optparse.OptionParser(usage)

  parser.add_option(
      "-e",
      "--eog",
      action="store_true",
      dest="eog",
      default=False,
      help = "Show the AST as an image using Eye-of-gnome. (assumes -i)" + S.OPTIONS_DEFAULT_STRING)

  parser.add_option(
      "-i",
      "--image",
      action="store_true",
      dest="image",
      default=False,
      help = "Show the AST as an image." + S.OPTIONS_DEFAULT_STRING)

  parser.add_option(
      "-n",
      "--no-database",
      action="store_false",
      dest="use_db",
      default=True,
      help = "Explicity tell redhawk to NOT use the database." + S.OPTIONS_DEFAULT_STRING)

  options, args = parser.parse_args(args)
  if len(args) != 1:
      parser.error("Exactly one file should be given.")

  database = S.GetDatabase() if options.use_db else None


  ast = G.GetLAST(args[0], database = database, key=S.GetKey(args[0], database))
  if options.eog:
    options.image = True

  if not options.image:
    print ast.ToStr()

  else:
    S.ShowASTAsImage(ast, options.eog)
  return



#!/usr/bin/env python
""" 
If you are an API user, you should almost always be using get_ast.py, rather
than this module.

This module contains functions that parse files and return asts. The
functions in this module should (generally) not be used directly. Instead, the
wrappers in redhawk/utils/get_last.py are to be used, as they also cache the
ASTs to store unnecessary parsing, and tree conversion.

      * GetLAST: Gets the Language Agnostic AST of a file. Essentially a
        function composition of ConvertAST and ParseFile.

      * ConvertAST: This function converts a language specific AST to the
        L-AST. This function takes an optional filename argument, to fill in
        the Module Node.



      The following functions are NOT required for external use:
      * ParseFile: This function delegates to the corresponding Parse
        function, using the language argument. If the language argument is
        None, utils.GuessLanguage is used.

      * ParsePython: This function parses Python files, and returns a
        python-ast. This function is generally not required for external use.

      * ParseC: This function parses C files, and returns a C-AST. This
        function is generally not required for external use.
"""

import redhawk.c.c_tree_converter as C
import redhawk.python.python_tree_converter as P
import util

import pycparser

import ast
import os

def GetLAST(filename, language=None):
  """ Parse the file using the respective parser, and return the Language
  Agnostic AST."""

  return ConvertAST(ParseFile(filename, language)
                    ,language or util.GuessLanguage(filename)
                    ,filename)


def ConvertAST(ast, language, filename=None):
  if language == 'c':
    converter = C.CTreeConverter(filename)
  elif language == 'python':
    converter = P.PythonTreeConverter(filename)
  else:
    raise NotImplementedError("Only C and Python Implemented so far")

  return converter.Convert(ast)




# ####
# Functions that return Language Dependent ASTs

def ParseFile(filename, language=None):
  """ Parse the file using the respective parser, and return a Language
  SPECIFIC ast."""
  language_parsers = {'c'       : ParseC
                     ,'python'  : ParsePython}

  language = language or util.GuessLanguage(filename)
  return language_parsers[language](filename)


def ParseC(filename):
  """ Parse a C file by running it through the preprocessor, and pycparser and
  return the C AST."""
  fake_libc_dir = os.path.join(os.path.dirname(__file__), "fake_libc_include")

  try:
    tree = pycparser.parse_file(filename,
        use_cpp = True,
        cpp_path='cpp',
        cpp_args='-I%s/'%fake_libc_dir)
  except StandardError, e:
    util.ExitWithError(str(e))
  return tree


def ParsePython(filename):
  """ Parse a Python file using the ast module and return the Python AST."""
  # print "FILENAME: ", filename
  try:
    tree = ast.parse(expr = open(filename).read(), filename = filename)
  except SyntaxError, e:
    util.ExitWithError(str(e))
  return tree



#!/usr/bin/env python

""" Tests conversion of full programs. """

import c_test_utils as CT

tests = [("prog001.c", "Function to return 0")
        ,("prog002.c", "Static function to return 0")
        ,("prog003.c", "An expression")
        ,("prog004.c", "Variable Declaration")
        ,("prog005.c", "Pointer Declaration")
        ,("prog006.c", "Pointer to array of chars")
        ,("prog007.c", "Assignment Statements")
        ,("prog008.c", "Function Call")
        ,("prog009.c", "Linked List Structure")
        ,("prog010.c", "If Else - Rec. Fibonacci")
        ,("prog011.c", "For Loop - Iter. Fibonacci")
        ,("prog012.c", "While Loop - Factorial")
        ,("prog013.c", "Do While Loop - Factorial")
        ,("prog014.c", "Switch Case - isspace")
        ,("prog015.c", "Cast - To Char *")
        ,("prog016.c", "Test Typedefs")
        ,("prog017.c", "Test Full Programs")
        ,("prog018.c", "Test Enum")
        ,("prog019.c", "Test Ellipsis Parameters")
        ,("prog020.c", "Test Break and Continue")
        ,("prog021.c", "Test Goto and Label")
        ,("prog022.c", "Test Union (decl and ref)")
        ,("prog023.c", "Test Structure Defintion")
        ,("prog024.c", "Test Ternary If")
        ,("prog025.c", "Test Hello World Empty arguments.")]

def ConvertFile(filename):
  t = CT.SetUp(filename)
  return CT.ConvertTree(t, filename=filename)

def TestPrograms():
  for (filename, description) in tests:
    ConvertFile.description = "Test %s (%s)"%(filename, description)
    yield ConvertFile, filename



#!/usr/bin/env python

""" A simple parser combinator library.
A parser is of type Parser :: s -> (object, remaining string) """

def Literal(literal):
  def NewParser(s):
    if s[:len(literal)] == literal:
      return (literal, s[len(literal):])
    return (None, s)
  return NewParser

def Regex(regex_object):
  assert hasattr(regex_object, 'match'), "regex object was not passed" 
  def NewParser(s):
    m = regex_object.match(s)
    if m:
      return (s[m.start():m.end()], s[m.end():])
    return (None, s)
  return NewParser

def Finished():
  def NewParser(s):
    if s == "": return ('', '')
    else: return (None, s)
  return NewParser

# Parser Combinators - Maybe, Many, Choice and Sequence
# If the parser fails, make it return a ''
def Maybe(p, empty = lambda: str()): # Universal Parser
  def NewParser(s):
    (m, r) = p(s)
    if m == None: m = empty()
    return (m, r)
  return NewParser

def OnePlus(p): # Do not pass a Universal Parser here!
  def NewParser(s):
    results = []
    old, (m, s) = s, p(s)
    if m == None: return (None, old)
    while m != None:
      results.append(m)
      old, (m, s) = s, p(s)
    return (results, old)
  return NewParser

def Choice(*parsers):
  def NewParser(s):
    for p in parsers:
      (m, r) = p(s)
      if m: return (m, r)
    return (None, s)
  return NewParser

# e_parsers are a list of (parser, error messages)
def Sequence(*e_parsers):
  def NewParser(s):
    results = []
    original_string = s
    for (p, e) in e_parsers:
      (m, s) = p(s)
      results.append(m)
      if m is None:
        if e: raise SyntaxError("%s: %s"%(e, s))
        return (None, original_string)
    return (results, s)
  return NewParser


# Clean the result of the parser p, by applying the function f
def Clean(p, f):
  def NewParser(s):
    (m, r) = p(s)
    if m: return (f(m), r)
    return (None, s)
  return NewParser




# Test With Statement

f()
fopen(filename)
with fopen(filename) as fp:
  x = fp.readline()
  y = fp.readline()
  s = fp.read()

# Open and close a file.
with fopen(filename):
  pass

with (fopen(filename1), fopen(filename2)) as (fp1, fp2):
  pass



#!/usr/bin/env python

""" 
This API is meant to be a private API. It is encapsulated by the `S` class in
selector.py which is to be used by the outside world.

A selector is a function that selects some nodes of the AST depending on
some constraints. Selector functions have the type

    selector :: tree -> Bool

Three types of selector functions are commonly required, and hence have easy
methods to create them (via the Selector function).

  1. Those that select depending on the type of the node.
  2. Those that select depending on the attributes of the node.
  3. Those that select if a function f, returns True on the node.

Selectors can also be combined. We choose styles similar to the CSS3
combinators (http://www.w3.org/TR/css3-selectors/#combinators):

  1. And(s1, s2) returns a new selector that selects a node only if BOTH
     s1, and s2 select the node.

  2. Or(s1, s2) returns a new selector that selects a node if EITHER s1 or s2
     select the node.

  3. HasChild(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node
        * s2 selects a child of the node.

  4. HasDescendant(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node
        * s2 selects some descendant of the node.

  5. HasParent(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node, n
        * s2 selects the parent of the node, n.parent.

  6. HasAncestor(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node, n
        * s2 selects some ancestor of the node, n.


Selectors are evaluted by calling RunSelector with the selector and the
tree.

Note that combining selectors is rather inefficient right now. This will be
fixed in the future.
"""

import redhawk.utils.util as U
import traverse


def RunSelector(s, trees):
  """ Runs a selector, `s`, on a list of trees, `trees`, (or a single tree,
  after converting it to a list) traversing it via DFS."""
  if type(trees) is not list:
    trees = [trees]
  for tree in trees:
    for node in traverse.DFS(tree):
      if s(node):
        yield node
  return


def Selector(node_type = None
            ,function = None
            ,attr_matcher = None
            ,match_only_common_attributes = True
            ,**attrs):
  """ Return a selector function that selects based on the criteria passed.
  
  * node_type takes a string, or a class, and checks if the node type matches.
  * function takes a boolean function, and selects if function(node) is true.
  * attrs is used to match the attributes of the node at hand.

  If match_only_common_attributes is True, we match only the common attributes
  passed in, and it is considered a failure only when some attribute does not
  match, or there are no common attributes. If false, we match all attributes,
  and the absence of an attribute would mean failure.
  
  In case a function, f, is passed and match_only_common_attributes is True,
  we simply do not match a node when an AttributeError is thrown when f is
  applied on the node. If match_only_common_attributes is False, we raise the
  AttributeError, and the program is expected to handle it (or comes to a
  grinding halt!)

  If `attr_matcher` is passed, a call to attr_matcher(node_attribute,
  argument) is used to check for equality between the attribute value passed
  in, and the attribute value in the node.

  If more than one criteria is passed, all the criteria have to be satisfied for
  a node to be selected."""

  def MatchNodes(node):
    # Node Type
    if node_type:
      if type(node_type) is str:
        match = node_type == node.GetName()
      else:
        match = isinstance(node, node_type)
    else:
      match = True

    if not match: return False


    # Function
    try:
      match = function == None or function(node)
    except AttributeError, e:
      match = False 
      if not match_only_common_attributes:
        raise AttributeError(e)
        
    
    if not match: return False


    # Attribute Matching
    if not attrs: return True

    count = 0
    for (key, val) in attrs.items():
      if match_only_common_attributes is False and hasattr(node, key) is False:
        return False

      if hasattr(node, key): 
        count = 1
        if attr_matcher:
          if not attr_matcher(getattr(node, key), val):
            return False
        elif val != getattr(node, key):
          return False

    return (count != 0)

  return MatchNodes


def And(s1, s2):
  """ And(s1, s2) returns a new selector that selects a node only if BOTH
     s1, and s2 select the node."""
  return lambda x: s1(x) and s2(x)


def Or(s1, s2):
  """ Or(s1, s2) returns a new selector that selects a node if EITHER s1 or s2
     select the node."""
  return lambda x: s1(x) or s2(x)


def HasChild(s1, s2):
  """ HasChild(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node
        * s2 selects a child of the node."""
  def MatchNode(x):
    if not s1(x):
      return False

    for child in U.Flatten(x.GetChildren()):
      if s2(child):
        return True
    return False
  return MatchNode


def HasDescendant(s1, s2):
  """ HasDescendant(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node
        * s2 selects some descendant of the node."""
  def MatchNode(x):
    if not s1(x):
      return False

    for node in traverse.DFS(x):
      if s2(node):
        return True
    return False
  return MatchNode


def HasParent(s1, s2):
  """ HasParent(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node, n
        * s2 selects the parent of the node, n.parent."""
  def MatchNode(x):
    if not s1(x):
      return False

    n = x.GetParent()

    if n is not None:
      return s2(n)

  return MatchNode


def HasAncestor(s1, s2):
  """" HasAncestor(s1, s2) returns a selector that selects a node, n, only if
        * s1 selects the node, n
        * s2 selects some ancestor of the node, n."""
  def MatchNode(x):
    if not s1(x):
      return False

    while x is not None:
      x = x.GetParent()

      if x and s2(x):
        return True

    return False
  return MatchNode



#!/usr/bin/env python

""" Miscellaneous Node Tests for the tree conversion. """

import c_test_utils as CT

def TestReturnConstant(): 
  """ Test `return 0`"""
  t = CT.SetUp("prog001.c")
  main = t.children()[0]
  return_node = main.body.block_items[0]
  return CT.ConvertTree(return_node)

def TestFileAST1():
  """ Test FileAST """
  t = CT.SetUp("simple_declaration.c")
  return CT.ConvertTree(t, "simple_declaration.c")

def TestFunctionDefinition1():
  """ Test Function Definition."""
  t = CT.SetUp("prog001.c")
  return CT.ConvertTree(t.children()[0])




""" Test the parser_combinator.py in redhawk/utils."""

import nose.tools
import redhawk.utils.parser_combinator as P
import re

digits = re.compile("[0-9]+")
bracketed = re.compile("{[^}]*}")


def TestLiteral():
  assert(P.Literal("a")("abcd") == ("a", "bcd"))
  assert(P.Literal("ab")("abcd") == ("ab", "cd"))
  assert(P.Literal("a")("bcd")  == (None, "bcd"))
  assert(P.Literal("abcd")("abcd")  == ("abcd", ""))

def TestRegex():
  assert(P.Regex(digits)("01231abcd") == ("01231", "abcd"))
  assert(P.Regex(digits)("abcd") == (None, "abcd"))

  assert(P.Regex(bracketed)("{asdfa}") == ("{asdfa}", ""))
  assert(P.Regex(bracketed)("{asdfa}gh") == ("{asdfa}", "gh"))
  assert(P.Regex(bracketed)("{}") == ("{}", ""))
  assert(P.Regex(bracketed)("a{}") == (None, "a{}"))

@nose.tools.raises(AssertionError)
def TestRegexWithNonRegexObject():
  P.Regex("foobar")

def TestFinished():
  assert(P.Finished()("") == ("", ""))
  assert(P.Finished()("foobar") == (None, "foobar"))

def TestMaybe():
  digits_parser = P.Maybe(P.Regex(digits))
  assert(digits_parser("asdf012") == ("", "asdf012"))

  digits_parser = P.Maybe(P.Regex(digits), lambda: list())
  assert(digits_parser("asdf012") == ([], "asdf012"))

def TestOnePlus():
  single_digit_parser = P.Regex(re.compile("[0-9]"))
  digits_parser = P.OnePlus(single_digit_parser)

  assert(digits_parser("01231") == (list("01231"), ""))
  assert(digits_parser("a01231") == (None, "a01231"))

def TestSequence():
  a, b, c = map(P.Literal, list("abc"))
  abc_parser = P.Sequence(
      (a, None),
      (b, None),
      (c, None))
  assert(abc_parser("abc") == (list("abc"), ""))
  assert(abc_parser("abcdef") == (list("abc"), "def"))
  assert(abc_parser("abdef") == (None, "abdef"))
  assert(abc_parser("foobar") == (None, "foobar"))

@nose.tools.raises(SyntaxError)
def TestSequenceFailure():
  a, b, c = map(P.Literal, list("abc"))
  abc_parser = P.Sequence(
      (a, None),
      (b, "b was expected to be found"),
      (c, None))
  abc_parser("acc")

def TestClean():
  int_parser = P.Clean(P.Regex(digits), lambda x: int(x))
  assert(int_parser("01213") == (1213, ""))
  assert(int_parser("5f") == (5, "f"))
  assert(int_parser("f5") == (None, "f5"))



#!/usr/bin/env python

import redhawk.common.node as N
import redhawk.common.types as T
import redhawk.utils.util as U
import writer

import itertools
import pygraphviz

def WriteToDot(tree):
  s = DotWriter()
  return s.WriteTree(tree)


def WriteToImage(tree, fmt='png', filename=None):
  s = DotWriter()
  s.AddTree(tree)
  s.Draw(path=filename, fmt=fmt)
  return


def EscapeWhitespace(s):
  a = s.replace("\n", "\\\\n").replace("\t", "\\\\t")
  return a


class DotWriter(writer.Writer):
  def __init__(self):
    self.node_name_counter = itertools.count(0)
    self.graph = pygraphviz.AGraph(directed=True, rankdir='LR')
    self.graph.layout(prog='dot')
    return


  def WriteTree(self, tree):
    """ Implementation of the base class method for writing the tree to a
        string."""
    self.AddTree(tree)
    return self.graph.to_string()

  def Draw(self, path, fmt='png'):
    self.graph.draw(path=path, format=fmt, prog='dot')
    return

  def AddTree(self, tree):
    """ Adds the tree to the graph."""
    self.AddASTNodeToGraph(None, tree)
    return


  def __CreateGraphNode(self, **attrs):
    """ Create a graph node with the give attributes."""
    node_index = self.node_name_counter.next()
    self.graph.add_node(node_index, **attrs)
    return node_index
    

  def __CreateGraphNodeFromAST(self, ast_node):
    """ Create a Graph Node (with the relevant attributes) 
          from the ast_node
        Return the node index."""
    name, attrs = ast_node.GetDotAttributes() 
    label = [name]
    label += ["%s: %s"%(EscapeWhitespace(str(k)), EscapeWhitespace(str(v)))
      for (k, v) in attrs.items() if type(v) is str]

    if isinstance(ast_node, T.Type):
      color = "gray"
      fontcolor = "blue"
    else:
      color = "gray"
      fontcolor = "black"

    return self.__CreateGraphNode(label = ", ".join(label)
                                 ,shape = "box"
                                 ,color = color
                                 ,fontcolor = fontcolor
                                 ,fontname = "Sans"
                                 ,fontsize = "10")


  def __CreateEmptyGraphNode(self):
    """ Create an Empty Node (with style), and return its index."""
    return self.__CreateGraphNode(shape='circle', 
                                style='filled',
                                label="",
                                height='.1',
                                width='.1')


  def AddASTNodeToGraph(self, parent_index, ast_node):
    """ Creates a Graph Node from the given AST node,
        marks its parent as the graph node with the given
        `parent_index`, and recurses on the given AST
        node's children."""

    node_index = self.__CreateGraphNodeFromAST(ast_node)

    if parent_index is not None:
      self.graph.add_edge(parent_index, node_index)

    children = ast_node.GetChildren()
    
    for child in children:
      if child is None:
        continue

      if type(child) is list:
        empty_node = self.__CreateEmptyGraphNode()
        self.graph.add_edge(node_index, empty_node)
        map(lambda a: self.AddASTNodeToGraph(empty_node, a),
              child)

      elif isinstance(child, N.Node):
        self.AddASTNodeToGraph(node_index, child)

      elif child is None:
        continue

      else:
        U.ExitWithError("%s's child (type: %s) was supposed to be a Node!\n %s"
          %(ast_node.GetName(),
            type(child),
            ast_node))

    return




#!/usr/bin/env python

""" Functions and Constants common to all the executables."""
import redhawk.utils.util as U

import string
import sys
import os
import tempfile

PYGRAPHVIZ_NOT_FOUND = """This feature requires the pygraphviz, which does not
seem to be installed.

It can be obtained using easy_install from http://pypi.python.org/pypi/pygraphviz
or from the distro's native package manager. pygraphviz goes by the name
python-pygraphviz on debian/ubuntu."""

# The standard Redhawk Database file.
DB_NAME = '.redhawk_db'

# The program name
PROGRAM_NAME = sys.argv[0]

# Default string passed at the end of help in optparse
OPTIONS_DEFAULT_STRING = "[default: %default]"

def ShowImage(filename, eog=False):
  if eog:
    os.system('eog %s'%(filename))
    return

  try:
    import Image
  except ImportError, e:
    print "Cannot find the Image module. Opening in your web-browser."
    webbrowser.open(filename)
    return
  
  im = Image.open(filename) 
  im.show()
  return

def ShowASTAsImage(ast, eog=False):
    try:
      import redhawk.common.writers.dot_writer as D
    except ImportError, e:
      ExitWithError(PYGRAPHVIZ_NOT_FOUND + "\n\nError: " + str(e))

    temp_name = tempfile.mktemp(suffix='.png')
    D.WriteToImage(ast, filename=temp_name)
    ShowImage(temp_name, eog)
    return


def MakeStringFromTemplate(template):
  """ Convert the template into a string, by substituting for $prog and
  $db."""
  return string.Template(template).safe_substitute(
      prog=PROGRAM_NAME,
      db=DB_NAME)

def ExitWithError(s, error_code = 1):
  """A quieter version of redhawk.utils.util's ExitWithError"""
  sys.stderr.write(s+"\n")
  sys.exit(error_code)

def IsFileSupported(filepath):
  """ Returns True if the file at `filepath` is supported by Redhawk. Else,
  returns False"""
  try:
    U.GuessLanguage(filepath)
  except KeyError, e:
    return False
  except ValueError, e:
    return False
  return True


def GetSupportedFiles(paths):
  """ Return a list of files, (after walking through directories), that are
  supported by redhawk."""
  for p in paths:
    if os.path.isdir(p):
      for root, dirs, files in os.walk(p):
        if '.git' in dirs: dirs.remove('.git')  # don't visit git directories
        for f in files:
          path = os.path.join(root, f)
          if IsFileSupported(path):
            yield path

    if os.path.isfile(p):
      if IsFileSupported(p):
        yield p



def GetDatabase():
  """ Return the location to the redhawk database if any."""
  try:
    return U.FindFileInDirectoryOrAncestors(DB_NAME, os.curdir)
  except IOError, e:
    sys.stderr.write("""The redhawk database exists, but does not have read & write
    permissions. Fix this to prevent re-parsing. Carrying on..""")
    return None


def GetKey(filepath, database):
  """ Return the key corresponding to the given filepath."""
  if not database: return None
  return U.AdjustFilePathToBaseDirectory(filepath, os.path.dirname(database))



#!/usr/bin/env python

""" Test the _selector module."""

import redhawk.common._selector as S
import redhawk.common.node as N
from sample_last import tree

import nose

""" The sample tree that we will test the selectors on:
(define-function Factorial 
  ((define-variable n))
  (compound 
    (if 
      (eq n 
        (constant 0))
      (compound 
        (compound 
          (return 
            (constant 1))))
      (compound 
        (compound 
          (return 
            (* n 
              (apply Factorial 
                ((- n 
                  (constant 1)))))))))))

"""

def TestEmptySelector():
  """ Test the Empty Selector (Should return all nodes)."""
  s = S.Selector()
  l = list(S.RunSelector(s, tree))
  assert(len(l) == 23) # Number of nodes in the tree.
  return


def TestSelector():
  """ Test Basic Selectors - function, attributes, node_type"""
  s1 = S.Selector(node_type = 'DefineFunction')
  s2 = S.Selector(value = '0')
  s3 = S.Selector(
      function = lambda x: hasattr(x, 'value') and x.value == '1')

  l1 = list(S.RunSelector(s1, tree))
  l2 = list(S.RunSelector(s2, tree))
  l3 = list(S.RunSelector(s3, tree))

  assert(len(l1) == 1 and len(l2) == 1 and len(l3) == 2)

  r1, r2, r3, r4 = l1[0], l2[0], l3[0], l3[1]

  assert(r1.name == 'Factorial')
  assert(r2.GetName() == 'Constant')
  assert(r3.GetName() == 'Constant')
  assert(r4.GetName() == 'Constant')
  return

def TestSelectorNodeType():
  """ Test a Basic Selector that takes a node type as a class."""
  s1 = S.Selector(node_type = N.Constant)
  l1 = list(S.RunSelector(s1, tree))
  assert(len(l1) == 3)
  assert(l1[0].GetName() == 'Constant')
  assert(l1[1].GetName() == 'Constant')
  assert(l1[2].GetName() == 'Constant')
  return


def TestMatchOnlyCommonAttributes():
  """ Test Matching only common attributes."""
  s1 = S.Selector(value = '0', asdf = 'Haha',
      match_only_common_attributes=False)

  s2 = S.Selector(value = '0', asdf = 'Haha',
      match_only_common_attributes=True)

  l1 = list(S.RunSelector(s1, tree))
  l2 = list(S.RunSelector(s2, tree))
  assert(len(l1) == 0)
  assert(len(l2) == 1)
  assert(l2[0].GetName() == 'Constant')
  return

def TestCommonAttributesFunction1():
  """ Test match common attributes for lambda function - success."""
  s1 = S.Selector(function = lambda x:x.value == '0',
      match_only_common_attributes=True)

  l1 = list(S.RunSelector(s1, tree))
  assert(len(l1) == 1)
  assert(l1[0].GetName() == 'Constant')
  return


@nose.tools.raises(AttributeError)
def TestCommonAttributesFunction2():
  """ Test match common attributes for lambda function - failure."""
  s1 = S.Selector(function = lambda x:x.value == '0' and x.asdf == 'Haha',
      match_only_common_attributes=False)
  l1 = list(S.RunSelector(s1, tree))
  return


def TestAttributeMatcherFunction():
  """ Test Attribute Matcher Function."""
  s = S.Selector(attr_matcher = lambda x, y: x.lower() == y.lower(),
                 name = 'fAcToRiaL')
  l = list(S.RunSelector(s, tree))
  assert(len(l) == 2)
  # Depends on order of traversal
  assert(l[0].GetName() == 'DefineFunction')
  assert(l[1].GetName() == 'ReferVariable')
  return


def TestAnd():
  """ Test And selector combinator."""
  s1 = S.Selector(node_type = 'Constant')
  s2 = S.Selector(value = '1')

  l1 = list(S.RunSelector(
              S.And(s1, s2),
              tree))
  assert(len(l1) == 2)
  return

def TestOr():
  """ Test Or selector combinator."""
  s1 = S.Selector(node_type = N.CallFunction)
  s2 = S.Selector(node_type = N.DefineFunction)

  l1 = list(S.RunSelector(
              S.Or(s1, s2),
              tree))
  assert(len(l1) == 2)
  return


def TestHasParent():
  """ Test HasParent selector combinator."""
  s1 = S.Selector(node_type = N.CallFunction)
  s2 = S.Selector(node_type = N.ReferVariable)

  l1 = list(S.RunSelector(
              S.HasParent(s2, s1),
              tree))
  assert(len(l1) == 1)
  assert(l1[0].GetName() == 'ReferVariable')
  return


def TestHasAncestor():
  """ Test HasAncestor selector combinator."""
  s1 = S.Selector(node_type = N.Return)
  s2 = S.Selector(node_type = N.CallFunction)

  l1 = list(S.RunSelector(
              S.HasAncestor(s2, s1),
              tree))
  assert(len(l1) == 1)
  assert(l1[0].GetName() == 'CallFunction')
  return


def TestHasChild():
  """ Test HasChild selector combinator."""
  s1 = S.Selector(node_type = N.CallFunction)
  s2 = S.Selector(node_type = N.ReferVariable)

  l1 = list(S.RunSelector(
              S.HasChild(s1, s2),
              tree))
  assert(len(l1) == 1)
  assert(l1[0].GetName() == 'CallFunction')
  return


def TestHasDescendant():
  """ Test HasDescendant selector combinator."""
  s1 = S.Selector(node_type = N.Return)
  s2 = S.Selector(node_type = N.CallFunction)

  l1 = list(S.RunSelector(
              S.HasDescendant(s1, s2),
              tree))
  assert(len(l1) == 1)
  assert(l1[0].GetName() == 'Return')
  return



#!/usr/bin/env python

import writer

import redhawk.utils.code_generator_backend as C

def WriteToScheme(tree):
  s = SchemeWriter()
  return s.WriteTree(tree)

class SchemeWriter(writer.Writer):
  def __init__(self, indent = '  '):
    self.code = C.CodeGeneratorBackend(tab = indent)
    return

  def WriteSExp(self, sexp, newline=True):
    if sexp is []:
      return

    if newline:
      self.code.NewLine()
      self.code.Indent()

    self.code.Write("(")

    for (i, s) in enumerate(sexp):
      if type(s) is list:
        if i == 0:
          self.WriteSExp(s,newline=False)
        else:
          self.WriteSExp(s)

      elif type(s) is str:
        self.code.Write(s)
        if i != len(sexp) - 1:
          self.code.Write(" ")

    self.code.Write(")")
    if newline:
      self.code.Dedent()
    return


  def WriteTree(self, tree):
    sexp = tree.GetRecursiveSExp()
    self.WriteSExp(sexp, newline=False)
    return self.code.GetCode()




#!/usr/bin/env python

import redhawk.common.writers.xml_writer as X
import common_test_utils as T

import nose.tools

import random
import itertools
import tempfile
import os

class TestXMLWriter:
  def __init__(self):
    self.counter = itertools.count(0)
    self.temp_dir = tempfile.mkdtemp(prefix='xml')
    return

  def GetFilename(self):
    i = self.counter.next()
    return os.path.join(self.temp_dir, str(i))

  def FunctionTestXML(self, ast):
    v = self.GetFilename()
    X.WriteToFile(ast, filename = v + '.xml')
    return


def TestGenerator():
  """ Testing XML Writer. """
  PICK=5
  c = TestXMLWriter()
  all_asts = list(T.GetAllLASTs())
  for i in range(PICK):
    r_index = random.randrange(0, len(all_asts))
    c.FunctionTestXML.im_func.description = "Test Random AST (%d) with XML Writer."%(r_index)
    yield c.FunctionTestXML, all_asts[r_index]


# Disable the test by default.
@nose.tools.nottest
def TestAllPrograms():
  """ Testing XML Writer (all programs) """
  c = TestXMLWriter()
  all_asts = list(T.GetAllLASTs())
  for (i, ast) in enumerate(all_asts):
    c.FunctionTestXML.im_func.description = "Testing AST (%d) with XML Writer."%(i)
    yield c.FunctionTestXML, ast




""" Type Classes.
    
    Used to structure the Type Heirarchy.
    
    This file is AUTO GENERATED from
      _types_cfg.yaml using _ast_gen.py
    The header is stored in _types_header.py
"""

import node

class Type(node.Node):
  """ The Base Type class."""
  pass





""" Definition of the NodePosition class."""

class NodePosition:
  def __init__(self, file, line, column):
    self.file = file
    self.line = line
    self.column = column
    return

  def GetFile(self):
    return self.file

  def GetLine(self):
    return self.line

  def GetColumn(self):
    return self.column

  def __repr__(self):
    if self.column:
      return "[%s:%4s:%3s]"%(self.file, self.line, self.column)
    else:
      return "[%s:%4s]"%(self.file, self.line)
  
  def __str__(self):
    return self.__repr__()




import redhawk.utils.util as U
import redhawk.utils.code_generator_backend as C

import pprint
import yaml

FILES = [# (input config file, out file, header file)
           ("_node_cfg.yaml", "node.py", "_node_header.py")
          ,("_types_cfg.yaml", "types.py", "_types_header.py")]

def GetClasses(file_body):
  """ Get a generator of classes, given the string of the body.
      This function guarantees that each class returned has the following
      attributes:
        * sexp
        * args
        * children
        * xml
        * json
        * dot
        * optargs
        * docstring
      All attributes except the docstring is a list.
  """
  nodes = yaml.load(file_body).items()
  nodes.sort()

  for (name, c) in nodes:
    U.AssertWithError(c.has_key('sexp'), "%s does not have sexp"%name)

    # Print appropriate warnings
    for w_key in "docstring args children".split():
      if not c.has_key(w_key):
        U.LogWarning('%s does not have %s'%(name, w_key))


    default_fields = {'docstring' : 'Represents a %s construct'%name
                     ,'args': ''
                     ,'children' : ''
                     ,'optargs' : ''
                     ,'super': 'Node'
                     ,'xml' : None 
                     ,'json': None 
                     ,'dot':  None }
              
    default_fields.update(c)
    c = default_fields

    for m in "args children optargs".split():
      if c[m] == None:
        c[m] = None
      elif c[m] == '':
        c[m] = []
      else:
        c[m] = c[m].split(", ")

    # pprint.pprint((name, c))
    yield (name, c)


def WriteMethodDeclaration(c, name, args, optargs):
  """ Write a method declaration:
    def name(self, args, optargs):
    in c """
  # Function Definition
  c.Write("def %s(self"%(name))
  for a in args:
    c.Write(", %s"%a)
  for a in optargs:
    c.Write(", %s = None"%a)
  c.Write("):")
  c.NewLine()
  return


def WriteList(c, li, prefix=None, frame=None, start_of_list=None):
  """ Write a list, li, into c.

      1) Add the Prefix prefix to every element of the list that is a
         variable (and present in the frame).
      2) Depending on whether the element is in the frame or not,
         add the string or the variable.
      3) Add start_of_list to the start of the list without making
      any modifications.
  """

  start_of_list = start_of_list or []
  prefix = prefix or ''

  elements = []
  for x in li:
    if (not frame) or (frame and x in frame):
        elements.append(prefix + x)
    else:
      elements.append("'%s'"%(x))

  c.Write("[")
  c.Write(", ".join(start_of_list + elements))
  c.Write("]")
  return


def WriteListReturnMethod(c, name, li, args):
  """ Write methods that return lists.
      The sexp type lists given can only be nested upto a depth
      of 2."""
  WriteMethodDeclaration(c, name, [], [])

  # If there is only one variable, and it starts with a *
  if len(li) == 1 and li[0][0] == '*':
    c.Indent()
    c.WriteLine("return self.%s[:]"%(li[0][1:]))
    c.Dedent()
    c.NewLine()
    return

  c.Indent()
  c.WriteLine("li = []")

  for a in li:
    if type(a) is str:

      # If a starts and ends with backquotes, we copy it verbatim.
      if a[0] == '`':
        U.AssertWithError(a[-1] == '`', "No closing backquote found at %s"%a)
        c.WriteLine("li.append(%s)"%(a[1:-1]))

      # Otherwise, we check to see if it begins with a '*', and take
      # appropriate action
      else:
        method, value = "append", a
        if a[0] == '*':
          method, value = "extend", a[1:]
        if value in args:
          c.WriteLine("li.%s(self.%s)"%(method, value))
        else:
          c.WriteLine("li.%s('%s')"%(method, value))

    elif type(a) is list:
      # If the first letter of the first variable is an underscore (a condition)
      # we write:
      # if self.condition:
      #   li.append(['a[1]', rest of list])
      if a[0][0] is '_':
        condition = a[0][1:]
        U.AssertWithError(condition in args, "%s not in %s, %s, %s"%(condition,
          name, li, args))

        c.WriteLine("if self.%s:"%(condition))
        c.Indent()
        if len(a) == 2 and a[1][0] == '*':
          extend_list = a[1][1:]
          assert(extend_list in args)
          c.Write("li.append([':%s'] + self.%s)"%(extend_list, extend_list))
        else:
          c.Write("li.append(")
          WriteList(c, a[1:], 
              prefix='self.', 
              frame=args, 
              start_of_list=["'%s'"%(':' + condition)])
          c.Write(")")
        c.NewLine()
        c.Dedent()
      else:
        c.Write("li.append(")
        WriteList(c, a, prefix="self.", frame=args)
        c.Write(")")
        c.NewLine()
  
  c.WriteLine("return li")
  c.Dedent()
  c.NewLine()
  return


def WriteAttributeMethod(c, name, li, args):
  """ Write the XML, JSON or Dot Methods into `c`."""
  WriteMethodDeclaration(c, name, [], [])
  c.Indent()
  c.WriteLine("d = {}")
  # c.WriteLine("d['tags'] = []")
  for x in li:
    if x in args:
      c.WriteLine("d[%s] = self.%s"%(x, x))
    # else:
    #   c.WriteLine("d['tags'].append('%s')"%(x))
  c.WriteLine("return (self.__class__.__name__, d)")
  c.Dedent()
  c.NewLine()
  return


def GenerateClass(name, attrs):
  """ Generate class `name` given attributes `attrs`."""
  c = C.CodeGeneratorBackend()
  c.WriteLine("class %s(%s):"%(name, attrs['super']))
  c.Indent()
  c.WriteLine('"""%s"""'%(attrs['docstring']))

  # Write __init__
  WriteMethodDeclaration(c, '__init__', attrs['args'], attrs['optargs'])
  c.Indent()

  args = attrs['args'] + attrs['optargs']
  for x in args:
    c.WriteLine("self.%s = %s"%(x, x))
  c.WriteLine("return")
  c.Dedent()
  c.NewLine()

  if attrs['children']:
    WriteListReturnMethod(c, 'GetChildren', attrs['children'], args)

  WriteListReturnMethod(c, 'GetSExp', attrs['sexp'], args)

  attribute_list = "xml json dot".split()
  function_list = "GetXMLAttributes GetJSONAttributes, GetDotAttributes".split()
  for (a, f) in zip(attribute_list, function_list):
    if attrs[a]:
      WriteAttributeMethod(c, f, attrs[a], args)

  c.Dedent()
  c.NewLine()

  return c.GetCode()


def GetYAMLFileAsPythonCode(filename):
  """ Return the YAML file as a string of Python Code."""
  return "\n".join(
      map(lambda a: GenerateClass(a[0], a[1]),
        GetClasses(open(filename).read())))


def Main():
  for (ip_file, op_file, header_file) in FILES:
    print(ip_file, op_file, header_file)
    op = open(op_file, "w")
    op.write(open(header_file).read())
    op.write("\n")
    op.write(GetYAMLFileAsPythonCode(ip_file))
  return 0

if __name__ == '__main__':
  Main()



#!/usr/bin/env python

""" Test XPath."""

import redhawk.common.xpath as X

import nose.tools

def RaisesSyntaxError(delayed_callable):
  assert(callable(delayed_callable))
  raises = False
  try:
    delayed_callable()
  except SyntaxError, e:
    raises = True

  assert raises, "Expression Does not Raise Syntax Error"


def TestStringParser():
  assert(X.string_parser("'foobar'") == ("foobar", ""))
  assert(X.string_parser('"foobar"') == ("foobar", ""))

def TestIdentifierParser():
  assert(X.identifier_parser("foobar") == ("foobar", ""))
  assert(X.identifier_parser("@foobar") == (None, "@foobar"))
  assert(X.identifier_parser("9foobar") == (None, "9foobar"))
  assert(X.identifier_parser("_foobar9") == ("_foobar9", ""))

def TestCodeBlockParser():
  assert(X.codeblock_parser("@{foo bar}") == ("foo bar", ""))
  assert(X.codeblock_parser("@{foo bar}fg") == ("foo bar", "fg"))

@nose.tools.raises(SyntaxError)
def TestCodeBlockParserFailure():
  X.codeblock_parser("@{}")

def TestNumberParser():
  assert(X.number_parser("1231") == (1231, ""))
  assert(X.number_parser("1231fg") == (1231, "fg"))
  return

def TestPositionParser():
  assert(X.position_parser("[1231]") == (1231, ""))
  assert(X.position_parser("[1231]fg") == (1231, "fg"))

  # Test Syntax Errors thrown up
  e1, e2 = False, False

  RaisesSyntaxError(lambda: X.position_parser("["))
  RaisesSyntaxError(lambda: X.position_parser("[1231"))


def TestAttrMatchParser():
  assert(X.attr_match_parser("@[foo='blah']") == (("foo", "blah"), ""))
  assert(X.attr_match_parser("@[foo='blah']f") == (("foo", "blah"), "f"))

  RaisesSyntaxError(lambda: X.attr_match_parser("@["))
  RaisesSyntaxError(lambda: X.attr_match_parser("@[foo"))
  RaisesSyntaxError(lambda: X.attr_match_parser("@[foo="))
  RaisesSyntaxError(lambda: X.attr_match_parser("@[foo='blah'"))
  RaisesSyntaxError(lambda: X.attr_match_parser("@[foo "))
  RaisesSyntaxError(lambda: X.attr_match_parser("@[foo= "))

def TestNodeQueryParser():
  assert(isinstance(
    X.node_query_parser("DefineVariable")[0],
    X.NodeMatchQuery))

  assert(isinstance(
    X.node_query_parser(
      "ForEach@[foo='blah']@{codegroup}[12]")[0],
    X.NodeMatchQuery))


def TestChildNodeMatchParser():
  assert(isinstance(
    X.child_node_match_parser("[DefineVariable]")[0],
    X.ChildNodeMatchQuery))

  assert(isinstance(
    X.child_node_match_parser(
      "[ForEach@[foo='blah']@{codegroup}[12]]")[0],
    X.ChildNodeMatchQuery))

  RaisesSyntaxError(lambda: X.child_node_match_parser(
    "[..]"))

  RaisesSyntaxError(lambda: X.child_node_match_parser(
    "[.]"))

  RaisesSyntaxError(lambda: X.child_node_match_parser(
    "[**]"))

def TestAtomicQueryParser():
  # Test this function rather thoroughly
  assert(isinstance(
    X.atomic_query_parser(".")[0],
    X.DotQuery))

  assert(isinstance(
    X.atomic_query_parser("..")[0],
    X.DotDotQuery))

  assert(isinstance(
    X.atomic_query_parser("*")[0],
    X.StarQuery))

  assert(isinstance(
    X.atomic_query_parser("**")[0],
    X.StarStarQuery))

  assert(isinstance(
    X.child_node_match_parser(
      "[ForEach@[foo='blah']@{codegroup}[12]]")[0],
    X.ChildNodeMatchQuery))

  assert(isinstance(
    X.node_query_parser(
      "ForEach@[foo='blah']@{codegroup}[12]")[0],
    X.NodeMatchQuery))

  assert(isinstance(
    X.node_query_parser(
      "ForEach@{codegroup}[12]")[0],
    X.NodeMatchQuery))

def TestSlashSepAtomicQueriesParser():
  result = X.slash_sep_atomic_queries_parser(
      "/./../*/**/[DefineVariable]/@[foo='blah']")[0]
  assert(isinstance(result[0], X.DotQuery))
  assert(isinstance(result[1], X.DotDotQuery))
  assert(isinstance(result[2], X.StarQuery))
  assert(isinstance(result[3], X.StarStarQuery))
  assert(isinstance(result[4], X.ChildNodeMatchQuery))
  assert(isinstance(result[5], X.NodeMatchQuery))

  RaisesSyntaxError(lambda: X.slash_sep_atomic_queries_parser(
    "/../"))


def TestParseXPath():
  # Thoroughly test final function
  query = ("./../*/**/[DefineVariable]/@[foo='blah']"
          + "/@{n.blah == None}/@{n.blah == None}")
  # No leading or trailing slash
  RaisesSyntaxError(lambda: X.ParseXPath(query + "/"))
  RaisesSyntaxError(lambda: X.ParseXPath("/" + query))

  _, leftover = X.xpath_query_parser(query)
  assert(leftover == "")

  result = X.ParseXPath(query)

  assert(isinstance(result[0], X.DotQuery))
  assert(isinstance(result[1], X.DotDotQuery))
  assert(isinstance(result[2], X.StarQuery))
  assert(isinstance(result[3], X.StarStarQuery))
  assert(isinstance(result[4], X.ChildNodeMatchQuery))
  assert(isinstance(result[5], X.NodeMatchQuery))
  assert(isinstance(result[6], X.NodeMatchQuery))
  assert(isinstance(result[7], X.NodeMatchQuery))
  return




#!/usr/bin/env python

""" Helper Functions for obtaining the position."""

def GetPosition(tree):
  """ Get the position of a node, of the position of its closest parent."""
  while (tree != None) and ((not hasattr(tree, 'position')) or (tree.position
    == None)):
    tree = tree.GetParent()

  if tree == None:
    return None

  return tree.position


def ShowPosition(position, context = 3, formatted=True):
  """ Show the position in some file."""
  (f, l, c) = position.GetFile(), position.GetLine()-1, position.GetColumn()
  assert(position != None)

  fp = open(f)
  lines = fp.readlines()
  fp.close()

  if formatted:
    return HighlightCurrentLine(l, lines, context)
  else:
    return "".join(lines[l-context:l+context])


def HighlightCurrentLine(l, lines, context = 3):
  line = "> " + lines[l]
  before = ["  " + x for x in lines[l-context:l]]
  after  = ["  " + x for x in lines[l+1:l+context]]
  if l < context:
    return line
  return "".join(before + [line] + after)



# Test Expressions
2
3 + 5
2 + 3 * 5
a + b * c
a << 2
(a - 2) * (b + c) /d
a % b
(c >> 3) * d
(a | b) ^ (c & d)
(a and b) or (c and d)
(a < b) and (c <= d) or (e > f) and (g > h)
(a == b)
(a != b)
not (a == b)
(-a)
(+a)
a < b < c
a <= b > c
a in x
a not in y
a is None
b is not None
(2, 3)
[5, 6]
"Redhawk"
"Redhawk" != x
a.x
b.y.z
x if condition else y
[2*x for x in li if x %2 == 0 if x%3 == 0]
(g for g in it)
(3*g for g in [1, 2, 3, 4])
lambda x: 2*x
lambda (x, y): x + y
lambda x, *y: y
li[0]
li[-1]
li[1:3]
li[:]
li[1:3:5]
f()
{2: '3', 'a': b}
2**3
Repr(2+3)



import redhawk.utils.util as U
import redhawk.utils.parse_ast as parse_ast

import sys

def ShowObject(a):
  for name in dir(a):
    if name[:2] != "__":
      print ("%10s : %s"%(name, str(getattr(a, name))))
  return

try:
  filename = sys.argv[1]
except IndexError, e:
  sys.stderr.write("No C file specified to parse.\n")
  sys.exit(1)

tree = parse_ast.ParseC(filename)
body = tree.children()
program = open(filename).read()
if "show" in sys.argv:
  tree.show(attrnames=True)
if "coord" in sys.argv:
  tree.show(showcoord=True)
else:
  U.StartShell(locals(), banner="Variables: tree, body, program, ShowObject")



#!/usr/bin/env python

import redhawk.utils.get_ast as G
import redhawk.utils.parse_ast as P

import os
import glob

AST_FILES = [("test/files/c/*.c", 'c', 'test/files/asts_c.pickle')
            ,("test/files/python/*.py", 'python', 'test/files/asts_python.pickle')]

def GetAllLASTs():
  for (g, language, database) in AST_FILES:
    for filename in glob.glob(g):
      yield GetLASTFromFile(filename, language, database)


def GetLASTFromFile(filename, language, database):
  """ We first fetch the tree's language specific AST, and use the ConvertAST
  function in parse_ast. This convoluted approach is taken since the language
  specific ASTs are cached by tests."""
  language_specific_tree = G.GetLanguageSpecificTree(filename
                                                    ,database
                                                    ,language=language
                                                    ,key = None)
  return P.ConvertAST(language_specific_tree
                        ,language
                        ,filename)


if __name__ == '__main__':
  files_found = list(GetAllLASTs())
  print "Files Found:"
  for x in files_found:
    print x, x.filename



#!/usr/bin/env python
""" Convert C-Tree to ast. """

import redhawk.common.node as N
import redhawk.common.node_position as NP
import redhawk.common.traverse as traverse
import redhawk.common.tree_converter as tree_converter
import redhawk.common.types as T
import redhawk.utils.util as U

# Map C operators into the LAST operators
BINARY_OPERATOR_CONVERSIONS = {
      '+'   : 'ADD'
     ,'-'   : 'MINUS'
     ,'*'   : 'MULTIPLY'
     ,'/'   : 'DIVIDE'
     ,'^'   : 'BITWISE_XOR'
     ,'|'   : 'BITWISE_OR'
     ,'&'   : 'BITWISE_AND'
     ,'<<'  : 'LSHIFT'
     ,'>>'  : 'RSHIFT'
     ,'%'   : 'MOD'
     ,'<'   : 'LT'
     ,'>'   : 'GT'
     ,'<='  : 'LTE'
     ,'>='  : 'GTE'
     ,'=='  : 'EQ'
     ,'!='  : 'NOT_EQ'
     ,'&&'  : 'BOOLEAN_AND'
     ,'||'  : 'BOOLEAN_OR'

     ,'.'   : 'ATTRIBUTE_INDEX'
     ,'->'  : 'ARROW' # TODO(spranesh): Bad Name?
}

UNARY_OPERATOR_CONVERSIONS = {
      '+'      : 'UNARY_PLUS'
     ,'-'      : 'UNARY_MINUS'
     ,'!'      : 'BOOLEAN_NOT'
     ,'++'     : 'PRE_INCREMENT'
     ,'p++'    : 'POST_INCREMENT'
     ,'--'     : 'PRE_DECREMENT'
     ,'p--'    : 'POST_DECREMENT'
     ,'*'      : 'POINTER_DEREFERENCE'
     ,'&'      : 'ADDRESS_OF'
     ,'sizeof' : 'SIZE_OR_LEN'
     ,'~'      : 'BITWISE_NOT'
}


def GetCoords(t):
  assert(t is not None)
  c = t.coord
  assert(c is not None)
  return NP.NodePosition(c.file, c.line, c.column)

class CTreeConverter(tree_converter.TreeConverter):
  def Convert(self, tree):
    """ Override the convert method in the base class to handle escape
    character representation by cpyparser."""
    l_ast = tree_converter.TreeConverter.Convert(self, tree)
    self.UnescapeEscapeCharacters(l_ast)
    return l_ast

  def UnescapeEscapeCharacters(self, tree):
    """ Cpyparser stores new lines as \\n, so that when printed they appear as
    \n. We want to unescape these newlines (and tabs), to match what python's
    ast module does."""
    for node in traverse.DFS(tree):
      for attr in node.GetAttributes()[1]:
        value = getattr(node, attr)
        if type(value) is str:
          unescaped_string = (value.replace("\\n", "\n").replace("\\t",
              "\t").replace("\'", "").replace("\"", ""))
          setattr(node, attr, unescaped_string)
    return

  def ConvertFileast(self, tree):
    return N.Module(position = NP.NodePosition(self.filename, 1, 1),
        filename = self.filename,
        children = map(self.ConvertTree, tree.children()))

  def ConvertReturn(self, tree):
    return N.Return(GetCoords(tree), 
        return_expression = self.ConvertTree(tree.expr))

  def ConvertConstant(self, tree):
    return N.Constant(GetCoords(tree), 
        value = tree.value, type = T.BaseType(tree.type))

  def ConvertId(self, tree):
    #TODO(spranesh): Is this assert always true?
    if tree.name == 'NULL':
      return N.Constant(GetCoords(tree), 
          value = tree.name, 
          type = T.Pointer(T.BaseType('NULL')))
    else:
      return N.ReferVariable(GetCoords(tree), 
          name = tree.name)

  def ConvertDecl(self, tree):
    # We have to check if the child is a 
    #   a. a function declaration
    #   b. a structure declaration
    # Otherwise it is a normal declaration
    t = self.ConvertTree(tree.type)
    if isinstance(t, N.DeclareFunction):
      t.name = tree.name
      t.storage = tree.storage
      t.quals = tree.quals
      return t
    if isinstance(t, N.Structure):
      t.storage = tree.storage
      t.quals = tree.quals
      return t
    if isinstance(t, N.Enumerator):
      t.storage, t.quals = None, None
      return t
    if isinstance(t, N.Union):
      t.storage, t.quals = None, None
      return t
    else: 
      return N.DefineVariable(GetCoords(tree), 
        name = tree.name, 
        init = self.ConvertTree(tree.init),
        type = t,
        storage = tree.storage,
        quals = tree.quals)

  def ConvertTypename(self, tree):
    #TODO(spranesh): Handle quals.
    t = self.ConvertTree(tree.type)
    return N.DefineVariable(None, # No coords for Typename
        name = tree.name,
        type = t)

  def ConvertIdentifiertype(self, tree):
    """ Returns Type Object """
    try:
      return T.BaseType(base_type = tree.names[0])
    except IndexError, e:
      # Default type is int in C
      return T.BaseType(base_type = 'int')

  def ConvertTypedecl(self, tree):
    """ Returns Type Object """
    # child is either an IdentifierType or a Struct
    t = self.ConvertTree(tree.type)
    assert(isinstance(t, T.BaseType) 
        or isinstance(t, T.EnumeratorType) 
        or isinstance(t, T.StructureType) 
        or isinstance(t, T.UnionType)
        or isinstance(t, N.Enumerator) 
        or isinstance(t, N.Structure) 
        or isinstance(t, N.Union))
    return t

  def ConvertPtrdecl(self, tree):
    # TODO(spranesh): Handle quals (such as constants)
    """ Returns Type Object """
    return T.Pointer(ptr_type = self.ConvertTree(tree.type))

  def ConvertArraydecl(self, tree):
    # TODO(spranesh): Handle array dimensions.
    """ Returns Type Object """
    return T.Array(array_type = self.ConvertTree(tree.type))

  def ConvertNonetype(self, tree):
    """ Handle cases when children are none."""
    return None

  def ConvertParamlist(self, tree):
    """ Handle Function Arguments. 
        In case the last is an ellipsis param, we store it as a var_arg
        of name va_list."""
    try:
      position = GetCoords(tree)
    except AssertionError, e:
      position = None

    #TODO(spranesh): Cheap Hack?
    if tree.params[-1].__class__.__name__ == 'EllipsisParam':
      try:
        va_list_position = GetCoords(tree.params[-1])
      except AssertionError, e:
        va_list_position = None

      return N.FunctionArguments(position = position,
          arguments = map(self.ConvertTree, tree.params[:-1]),
          var_arguments = [N.DefineVariable(va_list_position, 'va_list')])

    return N.FunctionArguments(position = position,
         arguments = map(self.ConvertTree, tree.params))

  def ConvertFuncdecl(self, tree):
    """ Handle Function Declarations."""
    if tree.args:
      arguments = self.ConvertTree(tree.args)
    else:
      arguments = N.FunctionArguments(position = [], arguments = [])

    return N.DeclareFunction(
        position = GetCoords(tree),
        name = None,
        arguments = arguments,
        return_type = self.ConvertTree(tree.type))

  def ConvertFuncdef(self, tree):
    """ Handle Function Declarations. Consists of body, and decl.
        body is a compound statement, and decl is a Declaration."""
    #TODO(spranesh): Handle param_decls, (K&R style of arguments)
    # int main(a)
    # char a;
    func_decl = self.ConvertTree(tree.decl)
    body = self.ConvertTree(tree.body)
    return N.DefineFunction(position = GetCoords(tree),
        name = func_decl.name,
        arguments = func_decl.arguments,
        body = body,
        return_type = func_decl.return_type,
        storage = func_decl.storage,
        quals = func_decl.quals)

  def ConvertCompound(self, tree):
    # The Case Node is screwed up. We need to flesh out the
    # case node's stmt, and put it up one level higher.
    compound_items = []
    for t in tree.block_items:
      ct = self.ConvertTree(t)
      compound_items.append(ct)

      # We could have a case of case of case of ...
      while isinstance(ct, N.CaseDefault):
        t = t.stmt
        ct = self.ConvertTree(t)
        compound_items.append(ct)

    return N.Compound(position = GetCoords(tree),
        compound_items = compound_items)

  def ConvertBinaryop(self, tree):
    assert(tree.op in BINARY_OPERATOR_CONVERSIONS)
    return N.Expression(position = GetCoords(tree),
        operator = BINARY_OPERATOR_CONVERSIONS[tree.op],
        children = map(self.ConvertTree, [tree.left, tree.right]))
    
  def ConvertUnaryop(self, tree):
    assert(tree.op in UNARY_OPERATOR_CONVERSIONS)
    return N.Expression(position = GetCoords(tree),
        operator = UNARY_OPERATOR_CONVERSIONS[tree.op],
        children = map(self.ConvertTree, [tree.expr]))

  def ConvertAssignment(self, tree):
    op = tree.op
    if len(op) is not 1:
      aux_op = op[:-1]
      #TODO(spranesh): Not make any transformations to the += like operators?
      assert(aux_op in BINARY_OPERATOR_CONVERSIONS)

      rvalue = N.Expression(position = GetCoords(tree.rvalue),
          operator = BINARY_OPERATOR_CONVERSIONS[aux_op],
          children = map(self.ConvertTree, [tree.lvalue, tree.rvalue]))
    else:
      rvalue = self.ConvertTree(tree.rvalue)

    return N.Assignment(position = GetCoords(tree),
          lvalue = self.ConvertTree(tree.lvalue),
          rvalue = rvalue)

  def ConvertFunccall(self, tree):
    arguments = []
    if tree.args:
      arguments = map(self.ConvertTree, tree.args.exprs)

    return N.CallFunction(position = GetCoords(tree),
        function = self.ConvertTree(tree.name),
        arguments = N.FunctionArguments(position = GetCoords(tree),
          arguments = arguments))

  def ConvertStruct(self, tree):
    # If the pycparser's structure's decls is None, 
    #     the structure is being referred to.
    # Else
    #     it is a structure declaration.
    # TODO(spranesh): Is pyparser a crazy hack or is something wrong with my
    # understanding?
    if tree.decls is None:
      return T.StructureType(structure_type = tree.name)
    else:
      return N.Structure(position = GetCoords(tree),
          name = tree.name,
          members = map(self.ConvertTree, tree.decls))
         
  def ConvertStructref(self, tree):
    op = tree.type  # a.b or a->b
    assert(op in BINARY_OPERATOR_CONVERSIONS)
    return N.Expression(position = GetCoords(tree),
        operator = BINARY_OPERATOR_CONVERSIONS[op],
        children = map(self.ConvertTree, [tree.name, tree.field]))

  def ConvertIf(self, tree):
    return N.IfElse(position = GetCoords(tree),
        condition = self.ConvertTree(tree.cond),
        if_true = self.ConvertTree(tree.iftrue),
        if_false = self.ConvertTree(tree.iffalse))

  def ConvertFor(self, tree):
    return N.For(position = GetCoords(tree),
        init = self.ConvertTree(tree.init),
        condition = self.ConvertTree(tree.cond),
        step = self.ConvertTree(tree.next),
        body = self.ConvertTree(tree.stmt))

  def ConvertWhile(self, tree):
    return N.While(position = GetCoords(tree),
        condition = self.ConvertTree(tree.cond),
        body = self.ConvertTree(tree.stmt))

  def ConvertDowhile(self, tree):
    return N.While(position = GetCoords(tree),
        condition = self.ConvertTree(tree.cond),
        body = self.ConvertTree(tree.stmt),
        do_while = True)

  def ConvertSwitch(self, tree):
    return N.Switch(position = GetCoords(tree),
        switch_on = self.ConvertTree(tree.cond),
        body = self.ConvertTree(tree.stmt))

  def ConvertCase(self, tree):
    return N.CaseDefault(position = GetCoords(tree),
        condition = self.ConvertTree(tree.expr))
    # Condition is None => default

  def ConvertDefault(self, tree):
    return N.CaseDefault(position = GetCoords(tree))

  def ConvertCast(self, tree):
    return N.Expression(position = None,
        operator = 'TYPE_CAST',
        children = [self.ConvertTree(tree.to_type).type,
                    self.ConvertTree(tree.expr)])

  def ConvertTypedef(self, tree):
    return N.DefineType(position = GetCoords(tree),
        name = tree.name,
        type = self.ConvertTree(tree.type))

  def ConvertExprlist(self, tree):
    return N.List(position = GetCoords(tree),
        values = map(self.ConvertTree, tree.exprs))

  def ConvertArrayref(self, tree):
    return N.Expression(position = GetCoords(tree),
        operator = 'INDEX_INTO',
        children = [self.ConvertTree(tree.name),
                    self.ConvertTree(tree.subscript)])

  def ConvertEnum(self, tree):
    # If the pycparser's Enum's values is empty:
    #     the enumerator is being referred to.
    # Else
    #     it is a enumerator declaration.
    # A crazy pycparser hack
    if tree.values is None:
      return T.EnumeratorType(enumerator_type = tree.name)
    else:
      return N.Enumerator(position = GetCoords(tree),
          name = tree.name,
          values = map(self.ConvertTree, tree.values.enumerators))

  def ConvertEnumerator(self, tree):
    return N.DeclareSymbol(position = GetCoords(tree),
        name = tree.name,
        value = self.ConvertTree(tree.value))

  def ConvertContinue(self, tree):
    return N.Continue(position = GetCoords(tree))

  def ConvertBreak(self, tree):
    return N.Break(position = GetCoords(tree))

  def ConvertGoto(self, tree):
    return N.Goto(position = GetCoords(tree),
        location = tree.name)

  def ConvertLabel(self, tree):
    return N.SourceLabel(position = GetCoords(tree),
        name = tree.name,
        statements = [self.ConvertTree(tree.stmt)])

  def ConvertUnion(self, tree):
    # If the pycparser's union's decls is empty:
    #     the union is being referred to.
    # Else
    #     it is a union declaration.
    # A crazy pyparser hack
    if tree.decls is None:
      return T.UnionType(union_type = tree.name)
    else:
      return N.Union(position = GetCoords(tree),
          name = tree.name,
          members = map(self.ConvertTree, tree.decls))

  def ConvertTernaryop(self, tree):
    return N.Expression(position = GetCoords(tree),
        operator = 'TERNARY_IF',
        children = map(self.ConvertTree, [tree.cond, tree.iftrue,
          tree.iffalse]))



#!/usr/bin env python

import redhawk.utils.parse_ast as parse_ast

import sys

def GetNodeTypeShorthand(node):
  cparser_type = ""
  if hasattr(node, 'type'):
    cparser_type = str(node.type).split(' ')[0].rsplit('.')[-1]
  # python_type = str(type(node)).split('.')[-1][:-2]  
  python_type = node.__class__.__name__
  if python_type and cparser_type:
    return "p%s-c%s"%(python_type, cparser_type)
  if python_type:
    return "p"+python_type
  return "c"+cparser_type

def GetNodeName(node):
  name = ''
  while hasattr(node, 'name'):
    name = node.name
    node = node.name
  return name


def NodeToString(node):
  if node.coord:
    position = '[%s] %d:%d'%(
      node.coord.file or '', 
      node.coord.line or 0,
      node.coord.column or 0)
  else:
    position = '[] 0:0'

  name = GetNodeName(node)
  if name:
    name = " '" + name

  return "%s%s"%(str(GetNodeTypeShorthand(node)).lower(), name)

def PrintTree(tree, indent_level = 0, fp=sys.stdout):

  fp.write("\n%s(%s"%(' '*indent_level, NodeToString(tree)))
  if len(tree.children()) == 0:
    fp.write(")")
  else:
    #fp.write("\n")
    for c in tree.children():
      PrintTree(c, indent_level+2)
    fp.write(")")

try:
  filename = sys.argv[1]
except IndexError, e:
  sys.stderr.write("No C file specified to parse.\n")
  sys.exit(1)

tree = parse_ast.ParseC(filename)

#tree.show(attrnames=True, showcoord=True)
PrintTree(tree)




#!/usr/bin/env python
import script_util as S

import redhawk.common.xpath as X
import redhawk.common.position as P
import redhawk.utils.get_ast as G

import optparse
import sys

usage = S.MakeStringFromTemplate("""$prog query [options] <query> file1 file2 ..

This command is used to query the ASTs.

For an introduction queries, see the redhawk homepage on PyPi -
http://pypi.python.org/pypi/redhawk

For a further discussion on queries, please read pydoc
redhawk.common.xpath (assuming redhawk is installed on your path).
""")

def Main(args):
  parser = optparse.OptionParser(usage)

  parser.add_option(
      "-C",
      "--context",
      dest = "context",
      type = "int",
      default = None,
      help = "Context to be shown." + S.OPTIONS_DEFAULT_STRING)

  parser.add_option(
      "-n",
      "--no-database",
      action="store_false",
      dest="use_db",
      default=True,
      help = "Explicity tell redhawk to NOT use the database." + S.OPTIONS_DEFAULT_STRING)

  options, args = parser.parse_args(args)
  if not len(args):
      parser.error("No query or files given.")

  parsed_query = X.ParseXPath(args[0])
  if len(args) == 1:
    sys.stderr.write("No files given\n\n")
    sys.stderr.write("Query was parsed as: %s\n"%(parsed_query))
    sys.exit(1)

  database = S.GetDatabase() if options.use_db else None

  files = args[1:]

  ast_fetcher = G.CreateLASTFetcher(database)
  for f in S.GetSupportedFiles(files):
    ast = ast_fetcher.GetAST(f, key=S.GetKey(f, database))
    results = list(X.ApplyParsedXPathQuery([ast], parsed_query))

    if options.context:
      # TODO(spranesh): Fix the context
      for r in results:
        p = P.GetPosition(r)
        print "+++%s:%d"%(f, p.line)
        print P.ShowPosition(p, context = options.context)

    else:
      fp = open(f)
      lines = fp.readlines()
      for r in results:
        p = P.GetPosition(r)
        if len(lines) != 0:
          print "%s:%d:%s"%(f, p.line, lines[p.line-1].strip())
        else:
          print "%s:%d:%s"%(f, p.line, "")
      fp.close()
  ast_fetcher.WriteDatabase()
  return



""" Node Classes.

    This file is AUTO GENERATED from
      node_cfg.yaml using _ast_gen.py
    The header is stored in node_header.py
"""

import writers.scheme_writer as S
import redhawk.utils.util as U

import copy
import pprint

# -1 means infinite number of operators possible
# A dictionary of allowed operators, and their arity.
ALLOWED_OPERATORS = {
     'UNARY_MINUS'         : ('-', 1)
    ,'UNARY_PLUS'          : ('+', 1)
    ,'SIZE_OR_LEN'         : ('size-or-len-of', 1)

    ,'MULTIPLY'            : ('*', -1)
    ,'ADD'                 : ('+', -1)
    ,'MINUS'               : ('-', 2)
    ,'DIVIDE'              : ('/', 2)
    ,'FLOOR_DIVIDE'        : ('//', 2)
    ,'POWER'               : ('power', 2)
    ,'MOD'                 : ('mod', 2)
    ,'LSHIFT'              : ('lshift', 2)
    ,'RSHIFT'              : ('rshift', 2)
    ,'BITWISE_OR'          : ('bitwise-or', 2)
    ,'BITWISE_XOR'         : ('bitwise-xor', 2)
    ,'BITWISE_AND'         : ('bitwise-and', 2)
    ,'BITWISE_NOT'         : ('bitwise-not', 1)

    ,'BOOLEAN_AND'         : ('and', -1)
    ,'BOOLEAN_OR'          : ('or',  -1)
    ,'BOOLEAN_NOT'         : ('not', 1)
    ,'EQ'                  : ('eq', 2)
    ,'NOT_EQ'              : ('not-eq', 2)
    ,'IS'                  : ('is', 2)
    ,'IN'                  : ('in', 2)
    ,'LT'                  : ('<', 2)
    ,'LTE'                 : ('<=', 2)
    ,'GT'                  : ('>', 2)
    ,'GTE'                 : ('>=', 2)

    # TODO(spranesh): Should we make this an if-else?
    ,'TERNARY_IF'          : ('ternary-if', 3) 

    ,'PRE_INCREMENT'       : ('pre-increment', 1)
    ,'POST_INCREMENT'      : ('post-increment', 1)
    ,'PRE_DECREMENT'       : ('pre-decrement', 1)
    ,'POST_DECREMENT'      : ('post-decrement', 1)
    ,'ADDRESS_OF'          : ('addr-of', 1)
    ,'POINTER_DEREFERENCE' : ('dereference-ptr', 1)

    ,'ATTRIBUTE_INDEX'     : ('.', 2)
    ,'ARROW'               : ('->', 2)
    ,'TYPE_CAST'           : ('cast', 2)

    # An Indexing operator (dictionaries and lists)
    ,'INDEX_INTO'          : ('index-into', 2)
}


def ExpandList(li, f):
  """ Recursively expands a list li as follows:
        For element e in the list, li:
          * If e is a list, call ExpandList(li, f) recursively.
          * If e is a Node, expand it (into a list),
              by calling ExpandList(f(e), f)"""
  if type(li) is not list:
    return

  for (i, e) in enumerate(li):
    if type(e) is list:
      li[i] = ExpandList(li[i], f)
    elif isinstance(e, Node):
      li[i] = f(e)
      if type(li[i]) is list:
        li[i] = ExpandList(li[i], f)
  return li


class Node:
  """A Parse Tree Node."""
  def __init__(self):
    raise NotImplementedError("Node is an Abstract Class.")
    return

  def __repr__(self):
    return self.GetName()

  def __str__(self):
    return self.GetName()

  def GetSExp(self):
    """ This function is not to be called directly. Code for this function is
    generated."""
    raise NotImplementedError("Base Node Class!")

  def GetChildren(self):
    """ This function is not to be called directly. Code for this function is
    generated."""
    return []

  def SetParent(self, parent):
    # We want __parent to be a private member, so that it doesnt arise in the
    # dot diagrams, or the other exported formats.
    self.__parent = parent

  def GetParent(self):
    return self.__parent

  def GetName(self):
    return self.__class__.__name__

  def MakeCopy(self):
    return copy.deepcopy(self)

  def GetAttributes(self):
    """ Return the lower case attributes of the class as a 
    pair - (class-name, dictionary-of-attributes). """
    #TODO(spranesh): Any extra attributes we are missing?
    d = {}
    # d['tags'] = []
    for x in dir(self):
      if 'a' <= x[0] <= 'z':
        d[x] = getattr(self, x)
    return (self.__class__.__name__, d)

  def GetRecursiveSExp(self):
    return ExpandList(self.GetSExp(), lambda x: x.GetSExp())

  def ToStr(self):
    return S.WriteToScheme(self)

  def GetXMLAttributes(self):
    return self.GetAttributes()

  def GetJSONAttributes(self):
    return self.GetAttributes()

  def GetDotAttributes(self):
    return self.GetAttributes()


class ControlFlowStatement(Node):
  """A base class for Control Flow statements."""
  def __init__(self):
    raise NotImplementedError("Base Class ControlFlowStatement not implemented!")

class ExceptionsStatement(Node):
  """A base class for Exceptions Related statements."""
  def __init__(self):
    raise NotImplementedError("Base Class ExceptionsStatement not implemented!")

